/**
 * =====================================================================
 * Copyright (C) 2009 Shawn E. Gano
 * 
 * This file is part of JSatTrak.
 * 
 * JSatTrak is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JSatTrak is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with JSatTrak.  If not, see <http://www.gnu.org/licenses/>.
 * =====================================================================
 */
package com.base.core.utiles;

/**
 * Astrodynamic Constants
 */
public class AstroConst
{

    /**
     * Modified Julian Date of J2000.0
     */
    public final static double MJD_J2000 = 51544.5;    // Modif. Julian Date of J2000.0
    /**
     * Gravitational Constant times Earth Mass [m^3/s^2]; JGM3
     */
    public final static double GM_Earth = 398600.4415e+9; // 9
    /**
     * Radius Earth [m]; WGS-84 (semi-major axis, a) (Equatorial Radius)
     */
    public final static double R_Earth = 6378.1363E3; // Radius Earth [m]; WGS-84
    /**
     * Arithmetic mean radius of the Earth = (2a+b)/3 [m]; WGS-84
     */
    public final static double R_Earth_mean = 6371008.7714; // Mean Radius Earth [m]; WGS-84
    /**
     * Semi-minor axis of the Earth = polar radius = b = (1?f)a [m]; WGS-84 (semi-major axis, a) (Equatorial Radius)
     */
    public final static double R_Earth_minor = 6356752.3142; // [m]; WGS-84
    /**
     * Radius Earth [m]; WGS-84 (semi-major axis, a) (Equatorial Radius)
     */
    public final static double R_Earth_major = 6378.1363E3; // Radius Earth [m]; WGS-84
    /**
     * Earth Flattening; WGS-84
     */
    public final static double f_Earth = 1.0 / 298.257223563; // Flattening; WGS-84 
    /**
     * Gravitational Constant times sun mass [m^3/s^2]; IAU 1976
     */
    public final static double GM_Sun = 1.32712438e+20;    // [m^3/s^2]; IAU 1976
    /**
     * Gravitational Constant times moon mass [m^3/s^2]; DE200
     */
    public final static double GM_Moon = GM_Earth / 81.300587;// [m^3/s^2]; DE200
    /**
     * Arcseconds per radian
     */
    public final static double Arcs = 3600.0 * 180.0 / Math.PI;     // Arcseconds per radian
    /**
     * Degrees per radian
     */
    public final static double Deg = 180.0 / Math.PI;          // Degrees per radian
    /**
     * Radians per degree
     */
    public final static double Rad = Math.PI / 180.0;          // Radians per degree
    /**
     * Astronomical unit [m]; IAU 1976
     */
    public final static double AU = 149597870000.0;      // Astronomical unit [m]; IAU 1976
    /**
     * Speed of light  [m/s]; IAU 1976
     */
    public final static double c_light = 299792458.0;         // Speed of light  [m/s]; IAU 1976
    /**
     * Solar radiation pressure at 1 AU. [N/m^2] (~1367 W/m^2); IERS 96
     */
    public final static double P_Sol = 4.560E-6;          // [N/m^2] (~1367 W/m^2); IERS 96
    /**
     * Difference in days between Julian to Modified Julian dates
     */
    public final static double JDminusMJD = +2400000.5;  // Difference in days between Julian to Modified Julian dates
    /**
     * EWG96 - spherical harmonics for  Earth gravity
     */
    public final static double[][] CS = {
        {1.00000000E+000, 0.00000000E+000, 1.54310000E-009, 2.68424890E-007, -4.49144873E-007, -8.08586948E-008, 2.06841175E-008, 6.98503096E-008, 4.05480290E-008, 1.40896376E-008, -8.11410140E-008, -1.58564025E-008, -2.46652822E-008, 2.12712874E-008, 1.54901114E-008, 5.57938101E-009, 1.60312924E-008, -1.46657305E-008, -1.80814610E-008, 5.74640504E-010, 2.95879626E-009, 1.22686232E-008, -1.59124408E-009, 6.65950295E-009, -7.16109670E-010, -3.16730604E-009, -2.99214279E-009, 1.43950291E-009, 2.97243229E-009, -3.61842391E-009, 4.50150898E-010, -5.72989567E-009, 4.48266424E-010, -7.47819338E-010, 2.17576429E-009, -3.47711651E-009, 1.70497068E-009, -3.65731664E-010, 1.51499877E-009, 1.71371576E-009, -5.26981171E-011, -1.28360232E-009, 6.34240989E-010, 4.26110701E-010, -2.40907557E-010, -1.63578141E-009, 2.99105637E-010, -4.39713795E-010, 8.69226480E-010, -6.76296205E-011, -4.22820806E-010, 8.62587487E-010, -1.56276859E-009, 2.41223100E-010, 7.38488414E-010, 1.35932032E-009, 8.98686574E-010, -7.61396236E-010, -3.47603752E-010, -5.61000426E-010, 5.84542468E-011, 2.94004066E-010, 8.54434057E-010, 4.07397608E-010, -1.26306399E-010, -1.02669514E-009, -3.86074932E-010, 4.45661394E-010, 7.00354529E-010, -6.74961461E-010, -6.87110263E-010, 5.63767446E-010, -3.40854157E-010, -1.94621445E-010, 2.80889126E-010, -2.85046614E-010, -6.17755606E-011, -1.01006523E-010, 5.00062896E-010, 1.62329011E-010, -1.12419502E-010},
        {0.00000000E+000, 0.00000000E+000, -9.03803807E-007, -2.11437612E-007, 1.48177868E-007, -5.23291936E-008, -4.64930357E-008, 9.26967114E-009, 5.37301949E-009, -2.23177405E-009, -3.06683731E-009, -5.12931944E-009, 1.46431604E-009, -2.54698406E-009, -1.88311945E-010, -1.01774873E-009, 8.65768669E-010, 1.78104177E-010, 2.52504297E-010, -3.06074718E-011, 3.78606608E-010, 7.61353535E-011, -3.96066477E-011, -8.82282205E-011, 2.82778998E-010, 1.40268168E-010, 1.68143690E-010, 6.98900808E-012, -1.05053721E-010, -3.29300581E-011, -3.11270489E-011, 7.22404148E-011, -3.62704694E-011, -5.18250818E-012, 5.23722216E-011, 7.07332073E-011, -3.08849489E-011, -1.19993373E-010, -2.44815636E-012, 5.67905571E-011, -8.62927707E-012, 1.32569435E-011, -1.38221975E-011, -4.73128565E-012, 1.59549492E-011, -1.85458036E-011, 1.06900505E-011, -6.11785933E-012, 2.15290493E-012, 1.72633616E-011, -1.77499788E-011, -5.54531551E-012, 2.78063824E-012, -3.33499536E-012, -7.62580697E-012, -9.82016876E-012, 2.28082775E-011, 8.69734377E-012, 2.14902575E-011, 4.50088823E-012, -8.91197448E-012, -6.03876431E-012, 1.96376207E-011, 2.69531174E-012, 1.23278537E-011, 1.82430988E-012, -2.96637911E-012, -1.60644070E-011, -5.06880452E-012, 6.08209826E-012, 4.34961805E-012, -1.03951616E-011, -2.29322845E-012, -6.65019305E-013, 4.12826899E-012, -7.96060213E-012, 5.82235822E-012, 5.99299849E-012, -2.26569971E-012, -1.10420098E-013, -5.11283557E-012},
        {-1.08262668E-003, -2.41400000E-010, 1.57446037E-006, 1.97222559E-007, -1.20077668E-008, -7.09734237E-009, 1.87163685E-010, -3.05943880E-009, -8.72954589E-010, -5.60998447E-010, -8.96536678E-010, -6.83255780E-010, 9.09866579E-011, 2.98330188E-010, 5.22039290E-011, 3.72366876E-011, -4.05148112E-011, 1.07431107E-011, -6.75548603E-012, 1.81542485E-012, 4.03189822E-011, 2.13170172E-011, 8.96959994E-012, -1.20915595E-011, -6.38993336E-012, -7.77060426E-012, 2.39890367E-012, 5.52116424E-012, 5.16469277E-012, -4.49807877E-012, -5.33761401E-012, -2.76134231E-012, 1.35218186E-012, 2.19253903E-012, 2.33042191E-012, 9.35400825E-013, -3.96130976E-012, -1.65325200E-013, -2.03495957E-013, 1.27650467E-012, -3.96988195E-013, 4.75140455E-013, 1.51627294E-012, -2.52980858E-014, -5.31365953E-013, -8.73082173E-013, 4.26173482E-015, 3.18597255E-013, 3.15550691E-013, 1.68140625E-013, -1.17792474E-013, -7.78634906E-013, -1.52749812E-013, 2.14381137E-013, 1.57323080E-013, 2.76100140E-013, 1.29082182E-013, 3.18869057E-013, -5.32286938E-013, -4.01495533E-013, 1.98553857E-015, 3.97495698E-013, 9.83763384E-014, -1.52381766E-013, -9.71456772E-014, 1.84615535E-015, -6.97614810E-014, 7.57332765E-015, 8.70561406E-014, -1.42316869E-013, -2.76357489E-014, -1.05987488E-014, 1.71823099E-013, 7.71773355E-014, 2.77797105E-014, -1.01205980E-013, -4.40487781E-014, -4.39616723E-014, 5.70675348E-014, 6.58356668E-014, 1.09891254E-014},
        {2.53265649E-006, 2.19263853E-006, 3.08989207E-007, 1.00548778E-007, 6.52571425E-009, 3.86712336E-010, -1.78450285E-009, -2.62893843E-010, 9.13943149E-011, 1.66584315E-011, -4.66932158E-011, -2.68075044E-011, 1.28845257E-012, -2.80786695E-012, -4.11179003E-012, 9.27377307E-013, 5.51541075E-012, 2.30398233E-012, -1.02477607E-013, -4.78432808E-013, -1.10985159E-012, 8.63706855E-013, 6.61412960E-013, 2.37058625E-013, 1.53739655E-013, 1.66285119E-014, -4.11498819E-013, 1.74903958E-013, 2.05030949E-013, 3.28278339E-014, -5.04910098E-014, -5.69766829E-014, -6.97188189E-014, 2.11005968E-014, 1.00044414E-014, 6.96576533E-014, 9.53225927E-015, 2.77814343E-015, -2.38052051E-014, -2.55420647E-014, -3.43679851E-014, 2.18308076E-014, 5.87017448E-015, 6.71977787E-015, -7.32792389E-015, 3.31543281E-015, -2.15025315E-014, 2.54347706E-015, -3.08690146E-015, 2.28413531E-014, 8.93560106E-016, 4.88250216E-015, 2.66968531E-015, 1.42820244E-015, -9.04415902E-015, 2.74610211E-015, 8.97861031E-015, -2.07826501E-015, 6.97361128E-017, 2.67310730E-015, 6.32066699E-018, -2.55444502E-015, -3.31193717E-016, 5.34383925E-015, 2.25002189E-015, -1.82793461E-015, -3.07062783E-015, 1.32559883E-015, -2.26163033E-016, 2.22945822E-016, 1.48855657E-015, -1.62527719E-015, -1.34063478E-015, -2.59503410E-016, 2.50511508E-015, -1.92702366E-016, -9.52873017E-016, -5.90405350E-018, -2.06147283E-016, -3.60279907E-016, -7.33327365E-016},
        {1.61962159E-006, -5.08799360E-007, 7.84175860E-008, 5.92099403E-008, -3.98407412E-009, -1.64818263E-009, -4.32981337E-010, 6.27778172E-012, 1.61353296E-011, -5.53453788E-012, -3.13743770E-012, 1.96838066E-012, 2.08290995E-013, 1.22628827E-012, -2.15565929E-013, 7.77370811E-014, -2.29399478E-014, 3.60489254E-014, 1.11601347E-013, 9.37452391E-014, -2.02942252E-014, 3.60882173E-015, 1.27689758E-015, -2.26830463E-016, -2.48286123E-014, -8.91909785E-016, 7.37658516E-015, 9.48281114E-015, -2.46183755E-015, 2.99406517E-015, -2.29188980E-015, 1.13955950E-015, 5.85503470E-016, 1.04651112E-015, 8.68160559E-016, -2.48115516E-015, 2.65856755E-016, -9.33749008E-017, 1.09890158E-015, 2.19911149E-016, -2.23620811E-017, -3.60727795E-016, -3.85248939E-016, 6.13352858E-016, 2.57977864E-016, -9.68970210E-017, -6.25717493E-016, -1.26771987E-016, -1.23366509E-016, -7.05592114E-018, -1.21922440E-016, -1.49991280E-016, -6.68602012E-017, -1.11195460E-016, -6.49505248E-017, 1.23198289E-016, 1.90382448E-017, 2.80238763E-017, -6.57719690E-017, -5.46568573E-017, -9.81274105E-018, -1.22017240E-018, 2.87924683E-017, 7.70823969E-018, -4.25828844E-017, -1.32321017E-017, 5.21814806E-017, 3.34002084E-017, 1.80736258E-018, 2.47218896E-017, 2.75661227E-017, 8.93101181E-018, 1.53449879E-017, 3.87364082E-018, -2.04436469E-017, 1.09826765E-017, -1.91412929E-019, 1.32641293E-018, 5.96567879E-018, -3.79451346E-018, 4.03657962E-018},
        {2.27296083E-007, -5.31803015E-008, 1.05587168E-007, -1.49300637E-008, -2.29930029E-009, 4.30822462E-010, -5.52609366E-011, 1.05356930E-011, 8.63661946E-012, 2.94330017E-012, -5.52270499E-013, 1.35820157E-013, 9.28981170E-014, -9.83140667E-015, 2.42978618E-015, -2.44367502E-014, -1.61912548E-014, -9.52652578E-015, -3.26112101E-015, 3.29525945E-015, -2.89868247E-016, -2.78268852E-019, -5.00859562E-016, 9.91939306E-016, 9.02658922E-017, 1.67260724E-017, -2.92522396E-016, 1.55958544E-016, 2.76364329E-016, 1.66699185E-016, 4.53346556E-017, 1.63295806E-017, -7.63972044E-017, -2.43013086E-017, 5.47895838E-017, 4.84065814E-017, -1.66066850E-017, 3.55744968E-017, 1.22664757E-017, 9.47349138E-018, 1.25599486E-017, 5.38210417E-018, -1.87427065E-018, 5.84534228E-019, 5.99353640E-018, -7.03462966E-019, -5.02785115E-018, -4.16558018E-018, 8.52833301E-018, 3.33743354E-018, 1.56133236E-018, -2.53286572E-018, -6.38822123E-019, 3.65853633E-020, -2.28458509E-018, -9.45297800E-019, 3.41607903E-019, -1.43382784E-019, 2.12883855E-019, -2.17130831E-019, -9.45279072E-019, -6.59467141E-019, -1.60405843E-018, 9.76506511E-019, -1.43941597E-019, 5.48691415E-020, -4.85037952E-019, 5.24525425E-021, 1.60471849E-019, 4.49378613E-019, 3.69112123E-020, -1.33046313E-019, -6.82728273E-020, 4.05877015E-020, -1.45435740E-020, 1.30863145E-019, 2.43253610E-019, 7.05771524E-020, -1.07528604E-019, -1.09255200E-019, 2.12222664E-020},
        {-5.40681239E-007, -5.98656699E-008, 5.99291239E-009, 1.18542073E-009, -3.26361778E-010, -2.15594107E-010, 2.25434971E-012, 4.53404227E-013, 3.81123970E-013, -1.83947129E-013, -2.83346836E-015, -3.73003245E-014, 7.90931969E-015, -7.85636339E-016, -3.15643361E-016, 2.50815727E-016, -2.68329714E-016, -8.54318009E-017, 9.50218985E-017, -6.90394120E-017, -1.53044141E-019, 2.19485809E-017, 1.60383714E-017, -3.04012550E-018, 9.67853726E-018, -6.48588226E-018, 5.86346598E-018, -3.67994784E-018, 3.84951576E-018, -1.28140382E-018, 8.80618690E-019, -2.66232294E-019, 8.66101818E-019, 6.41810564E-019, -1.39089637E-018, 8.27471280E-019, 9.02892800E-019, 1.06177943E-018, -2.86078252E-019, -1.43450046E-019, 1.76168554E-019, 2.05473616E-019, -9.77335772E-020, 2.25854393E-019, 4.15710622E-019, 8.94278421E-020, -2.28854217E-019, -1.02032273E-019, 5.61064074E-020, 6.76381516E-020, 7.84030244E-020, 4.77807330E-020, -2.66524199E-020, -6.91849850E-020, 1.75730606E-020, 3.21292032E-020, -7.14436106E-021, -2.96784806E-021, 1.71821433E-020, 8.65234271E-021, -1.34830740E-020, 5.95366561E-021, 1.03301202E-020, 9.33715751E-021, 2.16431358E-021, -1.20993343E-021, 7.51813861E-021, -5.12264488E-021, -1.09397151E-020, -1.11455258E-021, 3.02049038E-021, 3.86060905E-021, 6.66243069E-021, 2.05714240E-021, -2.00062227E-021, -5.42902767E-021, -2.43526919E-021, 1.56441130E-021, 1.08735050E-021, -1.17703711E-021, -1.87523423E-022},
        {3.52359908E-007, 2.04845995E-007, 3.28432703E-008, 3.52709152E-009, -5.84213577E-010, 6.85778101E-013, -2.49081644E-011, 2.02544297E-014, 1.53651295E-013, -1.00856971E-015, -1.05231479E-014, 1.16219383E-015, 3.76133302E-016, -1.17958445E-016, -9.35725610E-017, 7.70165144E-017, 1.03659054E-017, 4.59289071E-018, 3.40945629E-018, -5.31005717E-018, 6.56963210E-019, 3.70907385E-019, 8.31855442E-019, -4.03083806E-020, -3.83529569E-019, -5.05447623E-020, 5.52997370E-020, -3.23691023E-019, -9.08132318E-020, 2.32645225E-019, 7.48169045E-020, 2.88250733E-020, 4.88202027E-020, 1.14228844E-019, 2.95020775E-020, 4.66177629E-020, -2.06147285E-020, -7.15635969E-021, 1.18061819E-020, 2.58656997E-020, 7.08000115E-021, -6.53688947E-021, 8.30993168E-022, 3.01348698E-022, 3.51958037E-022, 3.49496912E-022, 2.06297944E-021, -6.84853075E-022, 2.58280099E-021, 1.50660499E-021, 3.99007940E-022, 1.29340161E-021, -2.72132911E-022, -6.19381441E-023, -4.31305208E-022, 6.67514094E-023, -5.39303848E-022, 1.29804820E-021, 6.10453604E-022, 1.32553929E-023, -1.00221496E-022, -5.73985179E-023, 1.01978566E-022, 2.08511574E-022, 5.64590266E-023, 7.15841901E-023, 2.85824541E-023, -9.72538272E-023, 3.94189250E-023, 7.93753233E-023, -1.87602911E-023, 8.65121979E-023, 1.80331436E-023, 9.93061494E-024, -6.35399857E-023, 1.99854396E-023, -1.60484049E-023, 3.95834066E-024, -3.62419238E-023, 2.68153390E-023, 7.34714544E-024},
        {2.04799467E-007, 1.60403960E-008, 6.59520391E-009, -1.93988889E-010, -3.19251714E-010, -4.62183966E-012, -1.83592366E-012, 3.43070291E-013, -1.58188596E-013, 7.44533054E-015, -6.99618085E-016, 2.56879338E-016, 6.12323874E-017, 4.86287506E-017, 1.49851811E-017, 1.00755630E-017, -5.78462417E-018, -2.35465436E-018, 1.82898830E-018, 2.11488622E-019, -1.27745254E-019, 1.03448688E-019, 7.21032507E-020, -7.14718152E-020, -6.25251449E-020, 5.71854875E-020, 9.07666549E-022, 1.52719547E-020, -1.04555004E-020, 1.05050863E-021, -5.37741000E-021, 2.18386433E-021, 2.27885220E-022, 1.80136382E-021, 3.95595080E-022, -1.59663580E-022, -4.31187478E-023, -1.70330538E-022, -5.68688269E-023, 2.17325795E-022, 9.21967653E-023, 1.30281679E-022, 8.14188757E-023, -1.78940005E-022, -1.33341636E-022, -7.96069966E-023, 8.42100007E-023, 3.49415515E-023, 3.94068313E-023, 4.60427135E-023, 1.76933356E-023, -2.18302215E-023, -2.45114919E-023, -9.14728176E-024, 2.89080915E-023, 6.33403380E-024, 2.36715700E-024, -1.14255945E-023, -1.23936519E-023, -7.25017007E-025, 3.18871378E-024, 5.27232946E-024, -1.78242625E-024, -6.15730175E-024, -2.46612821E-024, 1.27793181E-024, 3.16847097E-024, 1.88207682E-024, -5.22967339E-025, 2.66655471E-025, -3.06448024E-025, -3.69503307E-025, 9.24668243E-025, -2.45018388E-025, -2.99151633E-026, -3.09094261E-025, 4.63887009E-025, 2.47185989E-026, 4.36119725E-025, 1.46668720E-025, -1.43741578E-025},
        {1.20616967E-007, 9.31712334E-008, 1.53973453E-009, -1.21536485E-009, -7.70320134E-012, -1.69954645E-012, 8.27837204E-013, -2.25592760E-013, 6.15915977E-014, -3.67851284E-015, -9.97796839E-017, -1.73989230E-017, 9.19807581E-018, -4.12112625E-018, -6.83531529E-020, 3.22430989E-019, 1.23976352E-019, 1.03213900E-019, -1.45806514E-020, -1.32039855E-020, -5.42613111E-021, -9.49246002E-022, 8.97989717E-021, -4.80944680E-022, 3.55821169E-021, -4.74955645E-022, -4.98197017E-022, 9.21482353E-024, 3.05518380E-022, 4.76283383E-023, -7.83643970E-023, -6.56845055E-023, -5.81981558E-023, 8.10406695E-024, 8.31689900E-024, 4.91054943E-023, 2.23584458E-023, 6.02697789E-024, -1.22594567E-023, -4.22060415E-025, 3.64078115E-024, 1.43179702E-024, 5.11791497E-024, 1.13004604E-024, -2.90856088E-024, -9.36425910E-025, -2.33752995E-025, 9.37053797E-025, 6.32588013E-025, -6.60713562E-026, -5.03417229E-026, -3.09085527E-025, -2.24438293E-025, -3.28238058E-025, -7.52328640E-026, 2.15126595E-025, 5.87673985E-026, 1.39297507E-025, -7.76408125E-026, -1.52405157E-025, -6.53514934E-026, 2.44143389E-026, 1.03766021E-025, 2.89976358E-026, -3.80164612E-027, -2.11594576E-026, -4.18297982E-026, -7.04926534E-028, 1.50030673E-026, -1.33039468E-027, -4.45479409E-027, 1.47037359E-027, 4.21297692E-027, -1.76098461E-026, -6.46456962E-027, 7.79817551E-027, 2.39409255E-027, -4.67174428E-027, 1.40453386E-027, -1.20693531E-028, 2.37134356E-027},
        {2.41145439E-007, 5.16030047E-008, -5.60348353E-009, -4.02237710E-011, -4.95177735E-011, -3.06309899E-012, -2.60900996E-013, 6.83019997E-015, 4.63817277E-015, 2.33179880E-015, 4.17729666E-016, -1.40934700E-017, -2.80557607E-019, -6.28679155E-020, -1.84987078E-019, 3.47665347E-020, -2.58550775E-021, 4.33253137E-021, 4.00936289E-022, 1.08525155E-021, -1.12916333E-021, -1.21771467E-021, -3.63729773E-022, 1.88189632E-022, 1.44013056E-022, 5.28370224E-023, 6.74182419E-024, -2.03279693E-023, -2.05769970E-024, 6.09442699E-024, 7.16556084E-024, 9.74685421E-024, 2.51446782E-024, -2.06391159E-024, -1.86217018E-025, -3.94169194E-025, 4.44597734E-025, 2.57374796E-025, 4.69640193E-025, -9.54362723E-026, -1.26615171E-025, -1.58028329E-025, 5.01423201E-026, 8.38443884E-026, -1.21137971E-026, -2.58527469E-026, -2.27169852E-026, -2.58937672E-026, 4.70911379E-027, 7.89491176E-027, 5.54977275E-027, 4.65942514E-028, 1.69069201E-027, -6.38758392E-029, 9.51249998E-028, 2.80050518E-027, 2.68550865E-027, -5.38944679E-027, -2.66036379E-027, 5.95945669E-028, 1.77560770E-027, 1.31251441E-027, 7.22679075E-028, 7.05327160E-028, -1.03387104E-027, -2.33630342E-028, 9.84847347E-029, 8.72153375E-030, -3.15218978E-028, 2.49896361E-028, 2.93581964E-028, 1.04424405E-028, -1.27670388E-028, 3.96859089E-029, -7.85476064E-029, -3.17471035E-029, -3.75226363E-029, 1.68915922E-029, 2.93471439E-029, -7.29659697E-029, -2.40365835E-029},
        {-2.44402148E-007, 8.95449119E-009, 9.64619100E-010, -1.42927656E-010, -1.64036789E-011, 1.50332379E-012, -4.67520167E-015, 1.70904170E-015, -2.85040574E-016, -1.93232749E-016, -4.95199099E-017, 9.31276256E-018, -1.00345664E-019, 1.63896334E-019, -1.66969264E-020, 2.86940859E-021, 4.80753347E-022, 6.27087606E-022, -2.33137627E-022, 6.33944253E-023, 6.38973337E-023, 2.76379899E-023, -8.37335084E-024, -7.47810211E-024, -2.21286720E-024, 2.52355407E-024, 2.68469176E-025, 1.59615077E-025, 5.21729423E-025, -1.77537840E-025, -2.55721949E-025, 7.20009449E-026, 1.53589724E-025, 6.76591091E-026, -2.44261999E-026, -2.42340821E-026, -2.65967935E-026, -1.02701554E-026, -9.65576866E-027, 9.41487057E-027, 1.52503603E-028, 2.49700473E-028, -4.69820921E-027, 6.45242812E-028, -5.61831525E-028, -4.62388883E-028, -1.17733983E-028, 3.27703471E-028, -3.56776233E-028, -1.48553489E-028, 2.91091921E-028, 8.97935924E-029, -1.45737994E-028, -1.47495768E-028, -4.58596341E-029, 7.34211795E-029, 6.43635939E-029, 7.34259142E-029, -4.94915252E-030, -3.16586553E-029, -2.21188330E-029, -8.47257324E-030, 1.49655592E-030, -1.13063287E-030, 6.67752695E-030, 4.27580619E-030, -1.02283055E-029, -3.28158760E-030, 7.91669753E-031, 1.66719096E-030, 2.14121181E-030, 4.06205081E-030, 1.43321291E-031, -1.82672116E-030, -1.69241191E-030, -2.22248251E-033, 5.60683059E-031, 2.73206212E-031, 2.75150696E-031, 6.46393752E-031, -1.90861761E-031},
        {1.88626318E-007, -3.06085607E-008, 6.52283734E-010, 1.46760232E-010, -2.13223036E-011, 8.23572702E-013, 8.09843400E-015, -4.15107668E-015, -5.63595563E-016, 1.02472296E-016, -1.84220439E-018, 4.94768303E-019, -2.24005907E-020, 2.50751588E-020, 3.29808828E-021, -8.51180399E-023, 6.87828958E-024, 5.15078657E-023, -3.62037640E-023, -1.30549725E-023, 1.50725392E-024, 1.52282230E-024, 1.10767401E-024, -1.46647161E-025, 5.28098988E-026, -1.09286302E-025, 8.55977370E-027, -1.42110153E-026, 1.40128826E-026, -3.10427128E-027, 2.51543932E-027, 2.57172721E-027, 1.94498063E-027, 7.63819532E-028, 1.70609083E-028, 3.43403603E-028, 6.32680877E-028, -4.59933377E-028, -4.10911893E-028, -2.16968067E-028, -8.73085490E-029, 6.28140209E-029, 6.00871240E-029, -2.78862354E-029, -2.56462664E-029, -1.30422001E-029, -3.37777119E-030, -8.66056612E-030, 3.33161882E-030, 5.70274346E-030, 1.65439194E-030, -8.62028651E-031, 2.97140205E-030, 1.41032593E-030, 1.70261711E-031, -2.12627136E-031, 1.57457156E-030, -4.48795419E-031, -2.41224012E-031, -1.09588876E-031, -8.83191817E-032, -1.41858683E-031, -2.31779855E-031, 1.86898386E-032, 1.83318882E-032, 5.95368851E-032, 7.44404617E-032, -5.82239810E-032, -7.28278841E-032, -1.18728691E-031, -1.54560068E-032, -5.10815772E-033, 3.13516859E-032, 1.19488600E-032, 4.43467811E-034, -3.59362600E-032, -1.80456775E-033, 7.60235555E-033, -3.00016445E-033, -9.39794232E-034, -6.00263792E-033},
        {2.19788002E-007, -2.79743870E-008, 2.27041798E-009, -6.71317354E-011, -7.36453471E-013, 1.08811551E-012, -5.37038402E-014, 3.19834292E-016, -1.10722452E-016, 2.65575848E-017, 4.59359712E-018, -5.85618139E-019, -5.83302524E-020, -2.24221605E-020, -6.64118936E-023, -6.45469819E-023, -2.72421239E-023, 2.59785720E-024, -1.05681778E-024, -4.25259844E-025, -2.02470420E-025, 4.89484826E-026, 2.57760128E-026, -2.87932456E-027, -1.54402847E-027, 2.91823730E-027, 1.95751359E-027, 1.70026377E-027, -1.06761398E-027, -2.31618543E-028, 2.50160636E-028, 6.81236023E-029, 2.71075335E-029, 4.53479431E-029, 5.01688149E-029, -2.42692605E-029, -1.38024934E-029, -1.77761837E-030, 4.29654526E-030, 2.11072994E-030, 4.79269122E-031, -4.62525813E-031, 1.61332791E-030, 6.34448077E-031, -5.31087284E-031, -5.26630422E-031, 2.22385111E-032, 2.73195736E-032, -2.22219124E-032, -5.08098561E-032, 1.02316466E-031, 5.74772323E-033, -1.16394337E-032, -1.38950674E-033, 8.94742245E-033, 4.39968944E-033, 1.85073294E-032, 2.53525966E-032, -8.54489261E-033, -1.46230985E-032, -2.75231030E-033, 1.81029404E-033, 9.45066879E-033, 7.68233750E-034, -1.11626200E-033, -2.76490763E-033, -8.81581199E-034, 3.50825988E-034, -2.21275328E-034, 3.76205181E-034, 4.32283036E-035, -5.65196716E-035, -3.85383149E-034, -1.55544591E-034, 8.90134630E-035, -3.93772295E-034, -2.03992406E-034, -9.11839220E-035, 1.99369832E-035, -5.29498225E-035, -9.29373043E-035},
        {-1.30744533E-007, -9.82591637E-009, -1.34020695E-009, 9.15589749E-011, 3.33369243E-013, 3.77964272E-013, -1.90996003E-014, 2.83656685E-015, -2.13913329E-016, 1.66185804E-017, 1.84883350E-018, 7.29296869E-020, 4.50948813E-021, 2.35113059E-021, -7.15805238E-022, -2.27779823E-024, -2.93357299E-024, 1.22853565E-025, -1.44933989E-025, -3.53415913E-026, -7.65433658E-028, 4.26486215E-027, 8.35389896E-028, -3.06900672E-028, -6.63717904E-028, -1.56578956E-028, 9.04171513E-029, 7.20393203E-030, -6.77124468E-030, -1.33286068E-029, -1.20728820E-030, -1.94409121E-030, -3.69866463E-030, -9.14412538E-031, 1.04041666E-030, 9.33610391E-031, 2.31139916E-031, -4.30928727E-032, -1.00540290E-031, -8.19598717E-033, -8.06962131E-033, 2.97588898E-032, 4.29386326E-032, 2.84811230E-032, -2.15074262E-032, -3.60306759E-036, -4.04540984E-033, 1.26976957E-033, 1.60887783E-033, 7.65449080E-034, -8.88044851E-034, -2.67609320E-034, 4.87503879E-034, 8.86412481E-034, -6.24458343E-034, -6.49620825E-034, 1.42165235E-034, 2.67212717E-034, 5.00931609E-035, -1.61971211E-034, 4.48411549E-036, -3.57045187E-035, -8.35179529E-037, -2.45269169E-036, -1.28941319E-035, 1.35138421E-035, -1.71208671E-035, -8.41797894E-036, -2.43091056E-036, -1.88305896E-035, 6.14104307E-036, 3.11017068E-036, -3.21480578E-036, -6.35875816E-036, -5.83188535E-036, -1.46825817E-036, -1.80556308E-036, 2.57579398E-036, 1.16414241E-036, 8.81314013E-037, -6.53557532E-037},
        {8.23528409E-009, 5.12418911E-009, -7.04853993E-010, 1.12294621E-010, -5.82165257E-012, 1.20143889E-013, 2.19791555E-014, 2.81172327E-015, -1.12097927E-016, 3.45450289E-018, 2.29278604E-019, -2.14510302E-021, -5.98789707E-021, -5.72606577E-022, 1.37478590E-023, -9.23083960E-024, 4.78537983E-026, 1.03727472E-026, 4.68336420E-027, -1.54193407E-027, -2.01840658E-029, -1.79653070E-028, -7.99457995E-029, 5.31874295E-029, 6.52476351E-030, -1.34772300E-029, -3.19922843E-030, 7.51531729E-031, -1.94764085E-030, -1.16500105E-030, 1.91355998E-031, 1.54423844E-031, 6.32455402E-032, 5.56439192E-032, -8.05133270E-033, -2.45886405E-032, -3.36959218E-034, 1.79306727E-032, 8.45042421E-033, -1.05212497E-033, -1.24959618E-033, -1.08604151E-033, -4.07325335E-034, 3.24035817E-035, 3.91338900E-034, 2.63258746E-035, 2.15248085E-034, -2.17506595E-035, 1.93390701E-035, -8.58282824E-035, -5.21098963E-035, -1.31257430E-035, 1.20675915E-035, 2.50754961E-035, 4.39619012E-036, 2.72723575E-038, -6.04502218E-036, -5.62272478E-036, -3.18928540E-036, -5.17480463E-037, -5.20204420E-037, -1.47338505E-037, -1.11327800E-037, 6.89557043E-037, -3.70453548E-037, -7.76365745E-037, 1.40372601E-038, 1.43055098E-037, -2.85420111E-037, -2.73954567E-037, -3.39865899E-038, 5.90440760E-038, -5.59587403E-038, 3.86651670E-038, 1.54907214E-038, 1.85966626E-038, 9.30219792E-039, 1.28299147E-039, -1.32029219E-038, -1.54248239E-038, 7.57364712E-039},
        {-1.81139265E-008, 1.27266538E-008, -7.00504941E-010, -6.17631170E-011, 4.58612358E-012, -9.29938317E-014, 6.47350929E-015, -2.15621751E-016, -4.32877374E-017, -3.20104769E-018, -1.27578377E-019, 1.59971985E-020, 1.40841995E-021, 9.24857488E-023, -1.36277440E-023, -1.30039506E-024, -6.01304333E-025, -9.66670674E-027, 3.79176249E-028, -3.14253484E-028, -8.21609532E-029, -1.46221327E-029, -1.05958412E-029, -3.69975003E-030, -7.07281808E-031, -1.73822291E-031, 1.97009390E-031, 3.85980567E-033, -2.91346998E-032, -1.25797414E-032, -1.03372520E-032, 6.42482027E-033, 5.38782511E-033, 3.90994093E-033, 7.00350431E-034, -9.37372223E-034, -5.57744568E-034, -1.68112718E-034, 5.01421370E-035, -2.32255003E-035, 1.90552421E-035, 2.34481551E-035, -1.79208251E-035, -1.31608599E-035, 3.01328521E-036, -2.22672174E-036, -6.08754249E-037, 2.96544714E-036, 1.57714553E-036, -1.88793856E-037, -8.43155821E-037, -1.23079544E-037, 9.12678096E-038, 1.45668317E-037, 3.32290089E-037, -1.83051337E-038, -7.87132639E-039, -5.12215330E-039, 3.29848842E-038, 6.87512711E-041, 1.64395316E-038, 1.17533812E-038, -1.32560270E-038, -1.53466529E-038, -3.28116784E-039, -8.41740947E-039, -1.62339789E-039, -2.40319669E-039, -2.39457547E-040, -3.56487097E-039, -4.75326077E-042, 1.86891300E-039, 1.64720909E-039, -1.02579292E-039, -1.28111915E-040, 2.34396748E-040, 4.49832236E-040, -1.21257994E-040, -2.75121072E-040, -1.33625331E-042, -1.16870480E-040},
        {1.16904734E-007, -1.21569754E-008, -5.37628709E-010, 8.93438669E-012, 6.53457455E-013, -8.46504936E-014, -3.88575257E-015, 4.89961145E-016, 4.97088811E-017, 3.19029552E-019, -2.20976667E-020, -6.39778765E-021, 8.87732950E-022, 4.16515906E-023, -3.20887484E-024, 1.25042556E-025, -8.57436387E-026, -1.67059354E-026, -1.55013605E-028, -2.27140123E-029, -4.52693291E-031, -1.77436906E-030, -8.45212616E-031, -2.73819331E-031, -7.18105607E-032, -4.34756685E-032, 6.33977889E-033, 4.29888754E-033, -1.06174178E-033, -3.48922452E-034, -5.25232578E-034, -7.33333788E-035, -2.16464616E-035, -2.01517941E-035, -3.22008416E-035, -3.94293292E-035, 8.97543092E-036, 4.56187960E-036, -1.89934833E-036, -3.87205838E-037, 1.99114083E-037, 7.72094597E-037, 2.01306852E-037, -3.74891017E-037, -1.99429559E-037, -1.35601262E-037, -1.08996437E-037, 1.07272281E-037, 1.69763224E-038, -1.62881419E-038, -2.08271703E-038, -2.31743730E-039, -2.44192883E-040, 3.97598252E-039, -1.24852701E-039, 5.94247907E-039, 2.00268845E-039, 1.62955031E-041, -3.38595695E-040, 6.00968977E-041, -4.59922294E-040, -4.78925233E-040, 6.25900222E-041, 6.93585143E-041, 1.40684424E-040, -7.87580122E-041, -7.19650843E-041, 3.70583277E-041, 2.28979141E-041, -2.07949563E-041, -1.94251099E-042, -1.22980593E-042, 8.53012366E-042, 1.46386855E-041, -3.35271886E-042, -1.19844109E-041, -3.89956178E-042, 3.59788880E-043, -3.95218142E-042, 7.05326751E-045, -5.62851928E-043},
        {3.09424679E-008, 3.35426499E-009, 3.54769924E-010, -6.98072934E-012, 4.15736963E-012, 2.31660539E-014, 3.50331213E-015, 9.32603389E-017, 2.51016467E-017, -9.35943426E-019, 1.65027866E-020, -1.49852337E-021, -4.21565620E-022, -6.52528852E-024, -6.70063044E-025, -2.89605846E-025, 7.37344550E-027, 3.03611052E-028, 4.40547268E-029, 1.86582406E-030, 6.75282055E-031, 2.38053628E-031, -1.54497246E-032, 1.36586343E-032, -3.59376309E-033, 1.64970158E-033, 2.36837925E-034, -8.32530500E-035, 2.97227997E-034, 3.44657996E-035, 6.84748171E-036, 6.25426944E-036, -2.00504957E-036, 7.89913678E-037, 1.46965930E-036, -3.87824977E-037, -4.25500988E-037, -2.10772586E-038, -1.32235385E-038, 7.98906578E-038, 5.72198799E-039, -3.30789338E-039, -1.73271252E-038, -1.31469513E-038, -5.51325047E-039, -1.68324537E-039, -1.08595001E-039, 1.27292166E-039, 1.02357057E-039, -7.19149700E-041, 3.10848905E-042, -1.59927816E-041, 9.30123179E-041, -3.49336465E-041, -2.81604569E-041, -7.72122942E-042, 8.42788154E-042, 3.31104690E-041, -1.91928547E-042, -1.51102160E-041, -6.36688114E-042, 4.08955208E-042, 1.00233565E-042, 1.24316038E-042, 1.02734728E-042, -1.37149922E-043, -1.00254592E-042, 9.41714465E-044, 1.66021388E-043, 5.55224856E-043, -2.37905968E-043, -1.30779668E-043, -5.68657374E-044, -2.29150950E-044, -6.52340337E-044, 8.07093455E-044, 9.38300478E-044, -1.97590356E-044, -2.20547628E-044, 8.41264220E-045, 1.52004885E-044},
        {-2.03450148E-008, -3.44281882E-009, 8.23853590E-010, -1.17375905E-011, 9.86409937E-013, 3.62931731E-014, -7.23297998E-016, 4.60345875E-017, 1.57696065E-017, 9.38328117E-020, -6.05492964E-020, 1.74702881E-021, -1.70858267E-023, -3.44048101E-024, -1.54474607E-025, -4.44645073E-026, -4.61706990E-027, 5.94782981E-028, 8.29344219E-029, -9.93825512E-031, -1.20751774E-031, 2.59884526E-032, 7.14288953E-033, -5.16569781E-034, 2.64839134E-034, -5.69323420E-036, -4.35278543E-035, 5.08954669E-036, 4.03396828E-036, 1.52357386E-036, 1.54015402E-036, 3.21205382E-037, 8.87838965E-039, -1.08651804E-037, -7.35081496E-038, -4.34341743E-040, 7.30917284E-039, -1.16004952E-039, -1.35683724E-039, -3.69625590E-039, 1.52005025E-039, -8.84597181E-041, 4.99317055E-041, 1.68651001E-041, 1.85956151E-041, 7.66401460E-041, -5.23430957E-041, 1.48329254E-042, 2.47085559E-041, 9.82352600E-043, -5.01657069E-042, 4.43509183E-044, -2.74793164E-042, -9.69557295E-043, -2.23947079E-043, 3.43161786E-043, -2.62034544E-043, 3.21774648E-043, 1.48369579E-043, -1.08271297E-043, 2.47733757E-045, 4.62338884E-044, 9.65652474E-044, -1.22409172E-044, -2.90106944E-044, -7.69225029E-045, -7.38196549E-045, -1.21604297E-044, -4.33040285E-045, -2.48340722E-045, 1.17854689E-045, -2.64489845E-046, 3.71513880E-045, -3.49835984E-045, -5.35143219E-046, -2.98595776E-046, 4.38885416E-046, 2.10539629E-046, 4.60115081E-046, 2.23475802E-046, -4.99954595E-047},
        {1.42395629E-007, 2.28132434E-009, 4.29713024E-010, -3.85144314E-012, 1.27381504E-013, -2.81442960E-014, 1.47083840E-015, -1.43867155E-016, 1.58795777E-018, 3.44108288E-019, -3.44642771E-020, 8.36121852E-022, -2.25355465E-023, 6.00431742E-024, 1.63815053E-025, -2.53847510E-026, -9.02914719E-028, 2.61219606E-029, 8.50268579E-030, -1.99684784E-031, 4.02451959E-032, -9.29123403E-034, 9.11461534E-034, 9.69915869E-035, 3.10649561E-035, 5.44325913E-036, 3.73072500E-037, -5.66090834E-037, 1.94606345E-037, -6.90028910E-038, -3.19230323E-038, 1.68891464E-038, 1.28257819E-038, 4.31223471E-040, -1.93081502E-039, -1.07891085E-040, -3.89819635E-040, -5.51052583E-041, 7.80523062E-041, 2.14161134E-041, -4.01291573E-042, -1.38624810E-041, -1.17364631E-041, 6.15210129E-042, 1.48392092E-043, -2.54546741E-043, 1.36801951E-042, 1.95349392E-044, -3.74867089E-043, -4.17732855E-043, -4.04147223E-044, 2.81987111E-044, 1.52253949E-044, 3.41803351E-044, 2.04367607E-044, -1.81079864E-044, -1.45425176E-044, 3.55310824E-046, 8.86369868E-045, 1.90012660E-046, -4.29544755E-045, 1.92069730E-045, 7.08552336E-047, -5.63342415E-046, -8.12487132E-047, -6.70405302E-047, -2.80991282E-046, 1.00638201E-047, 1.83686513E-046, 1.58198672E-046, -4.10152759E-047, -5.30484132E-047, -2.05219098E-047, -1.28859408E-047, -3.96768162E-048, 4.04571820E-047, 2.85909629E-047, -6.31429975E-048, -1.06897319E-047, -4.73878798E-049, 6.11381123E-048},
        {3.85459517E-008, -6.94633816E-009, -1.31653603E-010, 1.84160243E-011, -2.56058119E-013, 5.46774502E-015, -1.42448875E-015, -4.28623747E-017, -4.19552658E-018, 2.04986233E-019, -7.10278035E-021, 2.40828912E-022, -6.05322708E-024, -2.05753929E-024, 1.31851464E-025, 7.16143076E-027, 2.15490981E-028, -1.38954527E-029, 4.12961361E-030, -3.97450043E-031, -4.30798419E-032, 2.05439674E-033, 1.45385393E-035, 3.83918029E-036, -8.08386972E-037, 1.74416295E-037, 1.07989608E-037, 1.35119309E-038, -1.20535427E-038, -9.14337054E-040, -2.31573688E-039, -1.22354600E-039, -1.23272513E-040, -3.20585415E-040, 3.24574052E-041, 2.67322648E-041, 3.75190442E-042, 3.43297072E-043, 4.89711820E-042, -2.68233562E-043, -2.85393140E-042, -3.21838683E-043, -1.94257441E-043, -6.45481687E-044, 3.97060076E-044, 5.70968896E-044, 8.08617103E-045, -4.00563311E-045, 7.12241439E-045, 7.06118166E-045, -4.97460296E-046, -1.71561307E-045, 1.52231831E-045, 1.59502663E-045, 6.06681784E-046, -2.55472501E-046, 2.81528999E-047, 6.46369630E-048, -1.05262477E-046, -8.55895142E-047, 6.23639825E-047, 2.57679799E-047, -9.05151123E-048, -1.86741871E-048, 5.94125962E-048, -4.45755645E-049, 3.53098457E-048, 6.43163544E-048, -2.57552056E-048, -9.12207044E-049, -6.16937289E-049, 1.39447447E-048, -2.67140840E-050, -2.04706929E-049, 3.61129410E-050, 1.63207321E-049, -5.71346789E-050, -8.42999853E-050, -9.64824662E-051, -6.64831054E-050, -7.10800941E-050},
        {-7.62958408E-008, 6.84527523E-009, -4.96115864E-010, 9.79421963E-012, -1.02427863E-013, -3.19692626E-015, 7.57903052E-016, 5.82706719E-017, -4.04439992E-018, 6.84751990E-020, 2.43144480E-021, -1.00903563E-022, 2.24555170E-024, -9.70526407E-025, 3.41367925E-026, 4.69114455E-027, 1.57232399E-030, 6.47090090E-030, 5.40492716E-031, 5.67729566E-032, -5.99952554E-033, -9.68574358E-034, -5.58354891E-035, -1.47996651E-036, -1.69212681E-037, -4.99987363E-038, 1.11026415E-038, -3.25416685E-039, 2.45300114E-040, 8.55343862E-041, -1.24436165E-040, 2.11937744E-041, -4.50186196E-042, -8.38205512E-042, -4.38658463E-042, -3.78394528E-043, 2.22570067E-043, 1.64910117E-043, 1.07906359E-043, 6.07471513E-044, -6.64492376E-044, -4.52026324E-044, -8.92484533E-045, -5.36220310E-045, 4.81848145E-045, 1.48149469E-045, 7.91652623E-046, 9.87003970E-047, -1.43174574E-046, -3.84149705E-047, -1.23855576E-046, 3.36680421E-047, 1.34746367E-047, -2.58154306E-047, -5.87308938E-048, 1.28497997E-047, 5.95716979E-048, -4.58392187E-049, 1.25072331E-048, 1.48540292E-048, -7.75304077E-049, -1.42845213E-049, -1.65190412E-049, 1.13484581E-049, -2.71772718E-050, 1.86308439E-049, 8.38371461E-050, -1.51574518E-050, 7.83664483E-051, 1.41728522E-050, -7.44016373E-051, -1.48694188E-050, -1.11144819E-050, 2.13132220E-051, 3.79362708E-051, 3.10267997E-051, -2.10061751E-051, -4.51719049E-051, -2.85147751E-051, 1.34970463E-051, 1.55181155E-051},
        {-1.55075644E-007, 4.57519253E-009, -2.37881634E-010, -1.84590635E-011, -7.77370648E-013, 1.12345764E-015, -7.75326369E-016, -2.18963186E-017, 9.21617106E-019, 1.42335457E-020, 4.28831782E-021, 1.16413601E-022, 9.86041879E-024, -3.48564173E-025, 1.06313312E-026, 1.59099461E-027, 2.95829806E-029, -1.59477219E-030, 1.54865164E-031, -6.59996498E-033, 8.31350151E-034, 1.29995245E-034, -1.58159299E-035, 3.73012932E-037, -1.05380091E-038, -3.40173494E-039, 1.23804946E-039, -3.71376204E-041, -7.89794253E-041, -4.33577769E-042, -1.70197648E-042, -7.47802407E-043, 5.62249782E-044, -2.96907290E-043, 6.13670610E-044, 4.08197994E-044, -1.23069525E-044, -1.18451912E-044, 7.48410589E-046, 2.50194110E-045, 6.26458015E-046, -1.36119868E-046, 5.40787258E-047, -4.12391516E-047, -8.60701697E-047, 3.43727961E-047, -1.26058027E-048, -2.72102299E-048, -1.30279359E-048, 1.18264855E-048, 2.64331035E-050, -1.88095248E-048, 6.43938575E-050, 6.56556846E-049, 3.15809705E-049, 1.82554545E-049, -6.74580216E-051, -4.63159136E-050, -2.04785811E-050, -3.33115913E-051, 6.27940596E-051, 1.20615412E-050, 2.37138927E-051, -4.95245416E-051, -3.23240408E-051, 7.90461777E-053, 1.83783753E-052, -8.95244110E-052, 2.40077214E-052, 4.51485692E-052, 2.45857183E-052, -3.91317137E-053, -1.01788937E-052, -7.53584591E-053, 3.66554959E-053, 1.67461228E-053, 4.43566628E-053, 1.98287216E-053, 1.53898128E-053, -9.60449035E-054, -1.22406864E-053},
        {5.34560170E-009, -1.27282973E-009, 2.29053101E-011, -3.23051419E-012, 1.69214609E-013, -8.47039690E-015, 2.20907547E-016, -1.26467135E-017, 1.36669272E-018, -3.75186443E-020, 1.85310860E-021, 1.11646476E-022, 4.22622428E-024, -4.88355389E-026, -1.64921986E-026, 2.54867198E-028, 1.95582840E-029, -1.45362980E-030, -4.62495636E-033, -1.93638849E-033, -1.53947614E-034, 1.33787178E-035, 7.42921154E-037, -1.19577122E-037, 3.48131799E-038, 2.73555139E-040, -4.94175044E-042, 9.28863435E-042, -7.04175521E-042, 9.61777027E-043, -5.25836705E-043, -2.49028324E-044, -2.58711741E-044, -1.59101686E-044, -5.92677789E-045, 4.97155985E-046, 1.67234346E-045, -9.48406099E-047, -4.97965463E-047, -6.56407655E-047, -2.11238794E-047, 7.72904620E-048, 5.59215165E-048, 2.14586355E-048, 3.93640326E-049, -6.50077571E-049, -8.43609353E-049, -5.85766077E-049, 1.18527794E-050, 1.03776362E-049, 3.81835841E-050, -6.13271652E-051, -7.67973515E-051, -1.56185434E-050, -4.36806196E-051, 1.04189744E-051, 1.50767363E-051, -7.67660083E-052, -5.77270151E-052, -2.82397555E-052, 5.37008267E-053, -1.50694665E-052, -2.47930218E-053, 1.31436724E-052, 1.15772992E-053, -3.25988190E-053, -1.02781556E-053, 2.84461669E-054, 1.07005992E-053, -2.36299383E-054, -1.38395329E-054, -1.48989719E-054, -9.08328931E-055, -1.04862096E-054, 7.76196016E-055, -6.54630804E-057, -2.94080088E-055, 1.72207990E-055, 2.09381221E-055, 2.44666548E-056, -2.74592105E-056},
        {2.29460671E-008, 2.73194310E-009, 3.41575527E-010, -7.22213881E-012, 2.28825328E-013, -9.72047663E-015, 6.48064359E-016, 1.21528847E-017, 2.04032790E-019, -8.04930023E-020, 1.00200198E-021, 6.03332994E-024, -1.64367733E-024, 8.04437798E-026, -8.80327946E-027, -9.28065389E-029, 9.65879912E-031, -8.23320506E-031, 3.55591352E-033, 1.29245917E-033, -7.59524560E-035, 7.15234856E-036, -6.80560269E-037, 3.40627721E-038, 1.68894669E-039, 6.22454989E-040, 2.22277417E-042, -3.57457390E-043, 1.22331723E-043, -5.16926219E-044, 2.51777189E-044, 9.72340201E-046, -7.36053265E-046, 3.16848285E-046, -3.34512566E-046, 4.64980400E-047, 3.84530245E-047, 1.82215113E-047, 4.69677889E-048, 2.99056612E-048, -7.46248931E-049, -6.52515139E-049, -2.36867106E-049, 6.79202221E-050, 1.24743917E-050, 3.01240974E-050, 2.96685626E-050, 4.31811431E-052, -4.72017525E-051, -4.43750556E-052, -4.66868036E-052, -5.48130736E-052, -2.59451008E-052, 1.38779868E-052, -6.58770445E-053, -2.89836710E-053, -5.40046046E-053, -3.36714811E-053, 7.48979700E-055, 2.67193555E-053, 6.93185753E-054, -4.87019291E-054, -1.50876095E-055, -1.37330132E-054, -5.82099493E-055, -2.59976288E-055, 6.16551679E-057, 3.03678304E-056, 4.66914518E-056, 1.35367430E-055, 3.30226545E-056, -5.34945323E-056, -6.86914104E-056, -1.87301426E-056, -1.82437637E-056, -3.07322337E-058, 8.66848260E-057, 3.19482135E-057, -5.53618866E-057, -2.86417747E-057, -4.31304941E-059},
        {3.68252445E-008, -6.01357002E-010, -5.26869314E-011, 7.82212210E-012, 4.03842668E-013, 8.69898091E-015, 3.54125253E-016, -1.90378147E-018, 1.89343513E-019, -2.29929290E-020, -1.09061557E-021, -1.65142854E-023, -2.32708113E-024, -1.90969198E-028, 1.98665283E-027, -1.48890680E-028, 6.75458694E-031, -3.14130485E-031, -1.65240313E-032, -1.37538218E-034, 2.44290450E-035, -1.92951623E-036, 1.45416705E-037, 1.26901163E-039, 7.16855584E-040, 3.25245025E-041, 4.50733542E-043, 1.51142465E-044, 3.90379720E-045, -5.87096531E-046, 1.71393429E-045, 3.85606609E-046, -7.08559107E-047, 4.29568874E-048, -4.57859572E-048, -5.70759794E-048, 1.44343961E-048, 2.88212221E-049, 2.38601672E-049, -2.37344512E-050, 8.82441628E-051, -9.79724470E-052, -2.67247940E-051, 4.96157457E-052, -7.55893121E-052, 4.72717411E-053, -9.33743683E-054, -9.83861392E-053, 9.50025795E-053, 2.98630459E-053, -1.50953113E-053, -2.87737597E-053, -1.19783447E-053, -3.47893068E-054, 3.28467647E-054, 1.35412617E-054, 8.23437913E-055, 9.52494190E-056, -4.16945833E-055, -7.49221747E-056, 6.33178827E-056, 1.57663107E-056, 2.69579037E-056, 9.18194272E-057, -1.24872687E-056, -9.51485445E-057, -3.91732466E-057, 7.70082795E-058, -1.67908553E-058, -7.60552273E-059, -2.35385961E-058, -1.12362118E-057, -9.32840597E-059, 3.36274657E-058, 1.41332237E-058, -1.22316908E-058, -6.56006561E-059, -4.58081425E-059, -6.31153416E-059, -3.89889553E-060, 2.70210980E-059},
        {2.05559462E-008, 9.49740553E-010, 2.01802316E-011, -1.83778897E-013, -1.11429028E-014, 1.14948255E-014, 9.26710491E-017, -1.17543885E-017, -2.19658136E-019, 4.67975062E-021, -7.09957482E-022, 4.16024175E-024, -9.54765139E-025, -1.70463244E-026, 2.21844604E-027, -1.10264703E-029, 7.43335395E-031, 4.64771969E-032, -1.65871504E-033, -9.26999934E-036, -1.25209324E-036, 3.88890634E-037, -2.67545482E-038, -1.59400126E-039, 8.60073142E-042, 2.02833750E-041, -1.05790762E-042, 1.65916442E-043, 2.72497728E-045, -3.08268279E-046, -8.11008901E-047, 6.55429529E-048, -3.50948405E-048, -1.81655151E-049, -1.12587900E-048, -2.91970261E-049, -2.95863016E-050, 1.02380915E-050, -5.25787472E-051, -4.43443432E-051, 1.24306774E-051, -3.65010208E-052, 6.91122056E-053, 1.29691098E-052, 4.32433396E-053, -5.84115090E-055, -1.00221283E-053, -6.60709054E-054, -3.72749595E-054, -2.96392557E-054, 7.90973160E-055, 5.15042973E-055, 7.16992588E-056, 7.87107598E-056, -1.11785076E-056, 1.32162787E-057, 5.53508918E-057, -1.29448439E-057, -3.78921286E-057, -1.89086102E-058, -8.90030726E-058, -7.01022181E-058, -2.13039030E-058, 4.07347803E-058, 3.52752934E-058, 2.86081863E-059, -8.92117254E-059, -4.56158417E-059, 7.13087201E-062, -1.80028413E-060, 9.53494213E-060, 3.45272845E-060, -2.64163521E-061, 6.38208853E-061, 9.73190963E-061, 1.73629468E-060, 2.42385628E-063, -1.77533796E-060, -1.02611263E-060, -6.39173476E-062, 3.35736804E-061},
        {-6.87319091E-008, -2.08135779E-009, -1.99969995E-010, 1.17408121E-012, -3.26615298E-014, 6.34267563E-015, -1.08789144E-016, -5.29736350E-019, -1.15693970E-019, 1.14270882E-020, -3.44748380E-022, -4.20293089E-024, -2.61403196E-026, 3.48301909E-027, -6.92458868E-028, -4.20851110E-029, -5.12071705E-031, 8.24886522E-032, 1.54239844E-033, 7.29885583E-035, -7.05382805E-037, 2.03812060E-037, -3.19080889E-039, 5.82402178E-040, 6.47567455E-041, 2.91802342E-042, 3.86824093E-043, -2.43305933E-044, 2.79785295E-045, -3.55359663E-047, 1.77726570E-048, -3.79806283E-049, 1.12968830E-049, 5.78430394E-050, -1.24725854E-050, 3.05323789E-051, 6.19326333E-053, 7.16812574E-052, 1.28047860E-052, -4.65893208E-053, 1.32405068E-053, 1.64288817E-053, -1.42654530E-054, 5.71809638E-055, 1.02464317E-054, -2.70387468E-055, -1.08830768E-055, 3.55396221E-058, -8.55135790E-056, -3.72851151E-057, 9.54232156E-057, 2.65336680E-057, -3.55801947E-057, -8.90117423E-058, 4.35779295E-058, -9.56099406E-059, -3.09355427E-058, -1.12039187E-058, -8.94963560E-059, -2.26091036E-059, 3.22744275E-059, 1.82313187E-059, 1.42661438E-059, 5.11895333E-060, -4.56538410E-060, -3.09608596E-060, 1.27550376E-060, 3.72530155E-061, -5.14274555E-061, -1.47484796E-061, 2.48404995E-061, 2.29767340E-061, -4.67166869E-062, 4.84362194E-062, 2.80754529E-063, -2.26336824E-062, -2.83873631E-063, 8.11845663E-063, 6.07797513E-063, 5.72017893E-063, 7.48305072E-064},
        {-3.82065136E-008, 1.83765389E-009, -3.90160026E-011, 7.76194626E-013, -3.52488703E-013, -3.39175419E-015, 2.05483623E-016, -3.52954214E-018, -3.54674747E-019, -9.58474482E-022, 3.63916055E-022, -5.73764985E-024, -1.61618546E-026, -1.44617288E-027, -1.63600873E-028, -1.88330505E-029, 1.07725911E-031, -5.05304092E-033, -4.96087220E-034, -3.70033023E-035, -2.11313188E-036, -1.23480183E-037, 7.15643999E-039, -5.99045727E-041, 6.22800682E-043, 6.48909982E-043, 5.87723407E-044, -4.59933452E-045, 5.25219341E-046, 9.08505140E-047, 9.84419560E-049, -1.22590214E-049, 4.00528875E-051, -1.17541925E-050, -9.48280987E-053, 1.30620706E-052, 8.41312873E-053, 5.72656441E-053, 3.43292388E-054, -6.08614353E-054, 2.34943737E-055, -3.80459030E-056, 5.30025294E-056, -1.28207403E-055, 1.24753194E-056, -3.61068920E-058, -1.12788592E-056, 1.67771598E-057, -4.50249780E-058, 4.69103704E-058, 5.31494735E-058, 1.41590467E-058, 9.68931988E-060, -7.49060064E-059, 9.39270697E-060, -1.90772340E-060, 3.72216221E-060, 3.29361721E-060, 2.32892325E-060, -4.46083178E-061, -5.38886786E-061, 1.23433610E-061, -1.19331235E-061, -9.92595705E-062, 2.70361483E-062, -5.97916929E-065, -2.18933418E-062, -7.38913420E-063, 4.76624644E-063, 3.43909322E-063, 1.76153211E-063, -1.75132409E-063, 1.46075100E-063, 1.07941454E-063, 4.87468482E-064, 2.57333872E-064, 2.81089701E-064, -2.10987878E-065, -6.64217314E-065, -1.37714516E-065, 3.74492959E-065},
        {4.70865961E-008, -2.01942354E-010, -1.23302026E-010, 8.39740186E-013, -5.87495890E-015, -1.87000172E-015, 4.68528639E-018, 1.93834632E-018, 4.12822038E-020, -4.04969026E-021, 8.15580849E-023, -6.92987900E-024, 4.04655105E-025, 1.20492429E-026, 1.58504156E-028, 2.43108952E-031, -4.78866587E-031, -1.05299204E-032, -7.62606669E-034, -3.66908857E-035, -5.90289077E-037, -5.67818859E-038, -1.19467244E-039, 6.88543057E-041, -1.45109107E-042, 1.03374281E-043, 2.47881367E-045, -1.06852761E-045, -5.57432577E-047, 3.90104540E-048, 3.20627420E-049, -3.70577298E-051, -5.59443422E-053, 4.70102260E-053, -1.61167924E-053, 1.01117939E-053, -3.84007328E-054, -1.92864058E-054, -5.17400732E-055, -2.60951648E-055, -1.16976589E-056, 3.31070769E-057, 5.96694608E-057, -4.35955669E-058, 5.60405102E-058, -4.49181445E-058, 3.62351482E-060, 1.78315020E-059, -3.37763207E-059, -2.60254165E-059, 1.21223283E-059, 5.19982216E-060, -1.06448324E-060, 7.11779017E-061, 6.22436790E-061, 1.09944564E-061, -2.44949626E-061, -5.81625564E-062, 1.81682519E-062, 2.80088277E-062, 1.86121670E-063, 5.84429052E-064, -4.53685037E-063, -1.29061337E-063, 2.08937202E-064, 2.29906994E-065, 9.30709315E-065, 4.18133681E-064, 2.04134985E-066, -5.25100340E-065, -4.05214273E-065, -1.30433277E-065, -1.50673852E-065, -2.12652169E-066, 6.88851659E-066, 6.78731938E-066, -1.46959898E-066, 1.32215121E-066, 6.42294615E-067, -3.86222516E-067, 3.74557676E-069},
        {5.81880157E-008, 2.17816624E-009, 8.49094372E-011, -3.21014626E-012, 1.41219099E-013, -3.22021219E-015, -2.01254536E-017, -1.05157585E-018, -9.48095198E-021, -2.73239953E-022, 5.53721775E-023, 3.23936030E-025, 8.80615899E-027, 5.18040656E-027, -1.52650535E-028, 2.97600051E-030, -1.81589195E-031, -2.38760728E-033, -2.16242540E-035, 4.07234068E-036, -1.00067244E-037, -2.30267999E-038, -9.36273952E-040, 3.90420095E-041, -6.29707627E-043, -1.85899869E-043, -7.73605985E-045, -4.84048220E-047, 2.43348681E-047, -2.75181067E-049, -1.49270496E-050, -1.71920650E-050, 4.56586775E-053, -1.77237455E-053, 1.64945562E-054, -1.13387746E-054, 8.66321810E-056, 4.09959183E-056, 5.43666888E-057, 5.37673069E-057, -1.88577870E-057, 4.52321349E-058, 2.25266948E-058, 8.95399607E-059, 5.36750983E-060, -8.32979845E-060, -3.73386425E-060, -8.28383536E-062, -4.08982151E-061, -6.24581196E-061, 1.07264226E-061, 5.41876807E-062, -6.50005662E-062, -1.11682539E-062, 5.59892926E-063, 2.10630956E-063, -2.14717872E-063, -6.40147390E-064, 3.66058143E-064, -2.07921888E-065, -2.43043414E-065, 2.31972356E-064, 1.49384466E-064, 3.50046144E-065, 2.09241829E-065, 6.53745431E-066, 2.50010544E-066, -4.81376870E-066, 1.44483355E-066, -3.73535905E-067, -1.19008963E-067, -1.18409351E-066, -4.02458654E-068, 3.71172865E-067, 9.16946660E-068, 3.68595225E-069, -3.24518568E-068, -5.93141797E-069, -1.56704691E-068, -1.41746420E-068, -8.31155320E-070},
        {-1.88629652E-008, -5.93699920E-010, 1.23203481E-010, -4.81750540E-014, 8.83964566E-015, 2.76048180E-015, -1.00523078E-016, 4.30931569E-019, 1.18967271E-019, 2.32403050E-021, 9.30195586E-025, -1.79723402E-024, -1.85746679E-025, 1.46231486E-027, -6.69563357E-029, 2.18428811E-030, 6.14095668E-032, -3.41962003E-033, 2.45708529E-034, 5.45563459E-037, 1.06031853E-037, -2.57000078E-039, -5.46450807E-040, 1.57807683E-041, -3.04838873E-043, -7.61443662E-044, 1.04272061E-045, -4.81235490E-047, 1.01221716E-048, 1.69648605E-049, -1.93962305E-050, -1.60487349E-051, 1.08750468E-052, 4.12433332E-054, 2.67870735E-055, -3.29316361E-056, -7.92664381E-057, -9.12760310E-057, 2.10142260E-057, 1.53538801E-058, -1.23368403E-059, 4.73966092E-059, 9.17892765E-060, -6.60082195E-061, 6.41444916E-062, -2.42429019E-061, -1.04134062E-062, 4.09831156E-062, -7.88595095E-064, -5.46950039E-063, -3.80775714E-063, 9.61812291E-064, -8.77023522E-065, 3.92408805E-064, 7.17516028E-065, -1.84272826E-065, 3.45874515E-065, 3.26050491E-065, 5.45636884E-066, -1.58061744E-066, 6.07168137E-066, -2.09969701E-070, -1.80313638E-066, -1.30336447E-067, 2.54844600E-068, -2.79271000E-068, -2.01638242E-067, 1.14052018E-068, -8.81929483E-069, 1.73501667E-069, 1.22601514E-069, 1.59175300E-068, 6.58737648E-069, -1.09124710E-069, -1.35258910E-069, -5.36096678E-070, -3.67334254E-070, -7.34039557E-070, 1.36228327E-070, 1.18217139E-071, -1.35802467E-070},
        {-2.85961176E-008, -4.82586170E-010, -7.73210652E-011, -6.17177639E-013, -3.96188231E-014, 6.64578882E-017, 1.03693730E-017, -1.57301710E-018, -6.48838792E-021, 1.22868697E-021, -1.81514904E-023, 5.79344217E-025, -1.76654579E-026, 6.33864595E-028, 3.91360795E-029, -1.22017599E-030, 6.54621588E-032, -1.73991621E-033, -1.03553817E-034, 3.24414259E-036, -4.64717020E-038, 4.95013702E-040, -2.16418836E-040, 9.58448830E-044, 3.85931273E-043, 7.95542164E-045, 7.64762910E-046, -4.52106437E-048, 3.13158075E-049, -1.64853269E-049, -9.99644925E-052, 2.15763398E-052, 2.64730813E-053, -7.54369574E-055, 1.23353970E-056, 2.42964054E-057, 6.80382586E-058, 1.68765512E-059, 2.55037922E-059, 6.93327045E-060, 5.73798626E-061, 7.45409688E-061, 6.72642202E-061, 8.67880466E-063, 4.45677640E-062, 1.73843931E-062, 4.50613446E-063, 1.66585720E-063, 1.36728285E-063, -4.33757082E-065, -9.74617489E-065, -1.39310196E-065, -8.91090156E-066, -2.62802334E-065, -6.35325712E-066, 8.38151975E-067, 4.37081950E-067, 3.48263326E-067, 2.20511116E-067, 7.69689542E-069, -7.09816565E-068, -4.29264538E-068, -2.35325040E-068, -8.77212043E-069, -3.55390130E-069, -2.86402484E-069, 1.39500903E-069, 9.98053343E-070, 7.33518709E-070, 3.18500598E-070, 7.81021466E-071, 4.83000798E-071, -2.88816320E-071, -1.00982536E-070, -1.50279180E-071, 1.26080107E-071, -7.94857508E-072, 2.85844357E-072, 3.85734060E-072, 3.17195716E-073, -2.33273880E-072},
        {-7.54933671E-008, -9.42916016E-010, 6.68760706E-011, 3.60149047E-012, -6.94368919E-014, -9.49282584E-016, 3.47629689E-018, 5.83106368E-019, -6.18957492E-020, 2.88072024E-022, -4.27145761E-023, -6.59771519E-025, 7.57263341E-026, -5.88195375E-028, -1.30110801E-029, 6.24529174E-032, -7.86231856E-033, -8.42749051E-034, -8.82437378E-035, -7.39322157E-037, 2.63113271E-038, -2.55471301E-040, -3.06766978E-041, -4.16505490E-043, 1.28164834E-043, 5.05423501E-045, 1.03426946E-046, 1.54009244E-047, -9.28933424E-051, 2.04662367E-050, -3.32276244E-051, -4.58307433E-053, 6.50848044E-054, 8.46525594E-055, -6.37051530E-056, -5.45731804E-058, -1.64468561E-058, -1.87734167E-059, 1.06313164E-060, 2.37991412E-061, -2.35707562E-061, 2.26469094E-062, -2.77967205E-063, 4.33362766E-063, -8.40103773E-064, 6.86649158E-064, 3.44116038E-065, 1.50382508E-065, 6.98978535E-066, 1.26341163E-065, 2.00196254E-068, 9.92002398E-067, 1.26717919E-066, -3.29000988E-067, -3.36077554E-067, -5.82067153E-068, 1.88842947E-069, -1.77832756E-068, -1.26355283E-069, -7.61372122E-070, 3.38690866E-070, 5.84477380E-070, -1.85902095E-070, 6.39916132E-071, 5.61593789E-071, 9.61697723E-071, -3.94677161E-072, -1.88626772E-071, -2.80324373E-072, 4.96128830E-072, -5.31484687E-072, -8.17479233E-073, -6.03106815E-073, -7.36522179E-073, 3.78456001E-073, -3.52833109E-073, -1.91692845E-073, 6.56526465E-074, 5.06738080E-074, -1.66489555E-074, -3.14625651E-074},
        {7.25022292E-008, -3.61324147E-009, -1.40239170E-010, 5.04157848E-013, -2.13615448E-014, -1.55490304E-015, 2.01376977E-017, -6.07068813E-019, 2.10408305E-020, -1.15011138E-022, -1.12887490E-023, 3.93971672E-025, 3.05333377E-026, -1.82410219E-028, -2.47449551E-029, -1.63800887E-030, -2.30183691E-032, 7.47158144E-035, -1.90854778E-035, -1.21516461E-037, 3.79640602E-039, 1.70657732E-039, 3.51018219E-041, -1.38875360E-042, 1.78005819E-044, 1.78390600E-045, -4.73858352E-047, 4.67492964E-048, 1.49745945E-049, 6.91888714E-051, -1.84175319E-052, 1.96203351E-053, -4.83393068E-055, 6.27190146E-056, -1.10799775E-057, -6.40058689E-058, -9.17580698E-060, -8.79316398E-061, -2.08790160E-062, -2.13449668E-062, 7.32258054E-063, -1.34248168E-063, -5.61574260E-064, -1.13650523E-064, -7.51841823E-065, 2.33392132E-065, -2.73453580E-066, -1.90582503E-066, -1.07978229E-067, 3.82251619E-068, 1.21556982E-069, 3.50031452E-068, 2.77012839E-068, 5.32032102E-069, -6.24671682E-069, 2.22949739E-069, -3.71372134E-070, -6.24449405E-070, -1.05925787E-070, 2.02918102E-071, -6.05000420E-073, 9.05500853E-072, 4.76189237E-072, -4.94776508E-072, -1.04830947E-072, -1.29266914E-072, -7.98862978E-073, -1.94374963E-073, 1.82969205E-073, 5.82358625E-074, -2.58863180E-074, -1.46227431E-074, -6.42082714E-075, 1.54899158E-075, 8.09565783E-075, 3.32297563E-075, -3.35405322E-076, -1.62758061E-075, 3.30500242E-076, 1.73127441E-076, -2.13984879E-076},
        {-3.43973563E-008, -3.75393181E-010, -3.91792106E-011, 1.74613825E-014, 2.06514322E-014, -1.40614402E-015, 5.69877198E-017, 2.49686736E-019, 1.41152937E-020, 3.36845337E-022, 1.38408997E-023, -3.83699209E-025, 1.31609298E-027, -4.84775509E-028, -2.41906133E-029, 6.62186466E-032, 2.63377536E-033, 3.19098580E-034, 3.56543829E-036, -3.35855955E-037, -1.25152687E-038, 7.33784271E-040, 7.47793006E-042, -2.92418383E-044, 6.10724696E-045, 4.66090796E-046, 1.51306152E-047, -1.29342703E-048, 1.51644788E-050, 5.51125394E-052, -1.39536911E-052, -5.81564949E-054, 4.72313251E-055, 8.82299334E-057, -1.41875115E-057, -1.81875132E-060, 7.10517171E-060, -9.24481626E-062, 5.15718475E-063, -1.85739359E-063, 1.34011759E-064, -2.04273931E-064, 1.13864481E-065, 7.95349120E-066, 2.94715328E-066, 4.80224005E-067, 2.56683820E-067, 2.38988583E-068, -2.38190671E-068, 1.83587685E-069, -1.09146521E-069, -1.76654493E-069, 3.85454091E-070, 3.75999852E-071, 4.21964670E-071, 6.34251840E-071, 2.67018393E-071, 6.54896201E-072, -3.23888244E-072, -1.21696281E-072, 1.81744400E-073, -1.50594346E-073, 1.38381708E-073, 1.55283719E-073, 3.12854044E-075, -1.29299966E-074, 1.02519206E-074, -1.95178290E-075, 4.63929497E-075, 4.63099610E-075, 7.67214843E-076, -3.93364903E-076, 7.55196626E-077, 5.71870655E-077, -1.40058168E-076, -6.13875708E-077, -8.88847281E-078, 1.22237088E-077, 9.03314893E-078, -6.02121627E-078, -8.52126467E-078},
        {-5.42892237E-008, -1.77790605E-009, -3.68074712E-011, -1.18161703E-012, 3.51642526E-014, -1.63312667E-015, -9.36965873E-018, 8.08191003E-019, -7.10093459E-021, 2.04294546E-022, 5.13061124E-024, 7.92293377E-026, -6.37619713E-027, -8.51061166E-030, -4.03248534E-030, 4.58985051E-031, 4.67777442E-033, 2.20039033E-034, 2.07774226E-036, -2.47992215E-037, -8.28043622E-039, 5.39257694E-041, 7.72481895E-042, -2.00434608E-044, -1.12343195E-044, 3.18436582E-046, 8.47478631E-048, -3.62528764E-049, 3.26547603E-050, 4.94723469E-052, -4.07514747E-053, 7.83535032E-055, -3.78705654E-058, 9.97061519E-058, 8.48270425E-059, -2.25700283E-059, -6.54656499E-061, 1.27853471E-061, -6.11271036E-064, 2.12684129E-064, 2.38150809E-065, -5.67869780E-067, -3.14321437E-066, 1.04913544E-068, -9.97857544E-068, 1.69720758E-068, -4.27896330E-069, 6.77488245E-070, -1.75907398E-070, -1.47729455E-070, -1.28714480E-070, -3.33871784E-071, -4.38301010E-072, -3.72170750E-072, -4.79574401E-073, 2.57323619E-073, 4.96752177E-073, 2.02366555E-073, 4.98903034E-074, 5.22880635E-074, -7.12587269E-077, -1.83363459E-074, 7.95072101E-076, -9.02590415E-076, -6.51100441E-076, 7.80836138E-076, 3.11301363E-076, 3.07739342E-078, -7.89794794E-077, -1.32914794E-077, -1.62280569E-077, -3.28465687E-078, 1.25573161E-078, 4.13298041E-078, -9.27668760E-079, -4.59470843E-079, 4.33218093E-080, 1.49547842E-079, 1.80290996E-079, 4.52017350E-080, 4.86141105E-080},
        {-2.19563266E-008, 9.97584223E-010, 6.86420719E-011, -5.51032802E-013, 2.88615596E-014, -1.37622399E-015, -5.46796844E-017, 1.58963255E-019, 8.55583891E-021, 3.37746811E-022, -3.44625270E-024, 1.49889629E-027, -5.16322451E-027, 5.84737974E-029, -7.50297449E-030, 8.30143553E-032, -4.66234079E-033, 9.69436874E-035, 7.29830518E-036, 2.13468154E-038, 2.61072851E-039, -3.22797792E-041, 6.39115770E-043, -3.54461713E-044, -8.34114718E-045, -5.93626882E-047, -2.08548266E-048, -5.24700338E-050, -4.88573535E-051, 2.47974480E-052, -2.02762923E-055, 2.61082144E-055, 2.52144019E-056, 3.72035357E-058, -7.42177328E-059, 2.18652265E-060, 3.51187586E-062, -8.49535506E-063, 8.75516830E-064, 4.94822956E-066, 1.35412284E-066, -2.08483209E-067, 1.36006117E-067, -7.09998548E-069, 1.96594191E-069, -1.61906778E-069, -3.86902328E-071, 1.36387543E-070, -5.02528438E-071, 4.45228152E-072, 6.42597801E-072, 1.85406493E-073, -1.04315064E-073, -8.62193905E-074, -2.80304357E-074, -2.85929270E-074, -1.04551930E-074, 1.69779004E-076, -7.77404575E-076, 2.55645833E-076, -6.17589538E-076, -1.54874480E-076, 1.08574446E-077, 2.05513704E-077, -2.10300266E-077, 2.88873487E-078, 7.78619363E-078, -4.23808659E-079, -5.88917961E-080, -8.98360470E-080, -3.19243783E-079, -3.82108074E-080, 1.33945457E-079, 5.10333165E-080, 9.58804374E-081, 1.76088175E-080, 1.72229149E-080, -3.17281207E-081, -3.59104870E-081, -9.61855978E-082, 2.95748347E-082},
        {1.54726296E-008, -8.18067520E-010, 2.23521948E-011, -6.79763867E-013, -2.86711685E-014, 7.24653766E-016, -1.51012396E-017, 1.52114615E-019, 5.18073436E-021, 4.06929410E-022, 9.39153159E-025, 5.92169041E-025, -4.28401480E-027, -5.48752824E-029, -2.76528207E-030, -1.13183211E-031, 8.23611616E-034, -1.33837701E-035, 1.21070939E-036, 6.53777794E-038, -2.40594794E-041, -5.69037126E-041, -2.23352404E-042, -3.30968566E-044, -3.28896099E-045, -5.34327220E-047, -1.32093412E-048, -9.69860982E-050, -1.17425658E-051, -3.29207435E-054, 2.54167372E-054, 3.31114067E-056, 2.99611180E-057, -2.39406044E-058, -4.09196771E-060, -1.45638613E-060, 2.35227583E-062, 5.41713428E-064, -4.94798164E-065, -3.26943326E-066, -1.07468757E-068, -2.57842698E-068, -2.45270071E-069, 8.18536146E-072, 2.66984940E-070, -2.93338491E-071, -2.77152445E-072, 5.31928737E-072, 1.01657699E-072, 1.25695437E-073, 1.40853213E-073, -3.06952699E-075, -5.22582853E-075, -1.26296689E-075, 2.96469784E-076, -7.81850915E-076, 2.65261782E-076, 2.21934936E-077, -5.35693964E-077, 9.33042487E-078, 2.39734424E-078, -3.04136682E-078, -1.63156464E-078, 6.61806354E-079, 1.41635351E-079, -8.18768070E-081, -9.33953503E-080, -3.98921650E-081, -9.16994209E-081, -1.64684421E-080, 1.34214404E-081, 1.55851997E-081, -2.95553075E-083, -3.60814600E-082, -5.62564270E-082, 5.68715968E-083, -1.04083146E-082, -9.29442421E-084, -3.18662891E-083, -6.60079867E-084, -7.70205663E-084},
        {-6.02741647E-008, 9.36969684E-010, -4.28066609E-012, 1.93968946E-014, 2.70142504E-014, 1.58836432E-015, -1.02725506E-018, -1.82496800E-019, 1.26155174E-020, -3.23127154E-023, -6.23805374E-024, 1.05652886E-025, 5.12262670E-027, -5.06033149E-029, -2.83016370E-031, -6.97045965E-032, -3.04164205E-034, 1.14949211E-035, -1.22986868E-037, -1.78513920E-038, -1.71919363E-039, -2.75079795E-041, -2.67118840E-042, -3.37245026E-045, 6.60636261E-046, 2.05915767E-048, 1.09105525E-048, -1.03572921E-050, 3.57849579E-052, 1.50327075E-053, 2.46262641E-056, -5.05476423E-056, -7.92130010E-058, -2.03989447E-059, 4.02768399E-061, 2.13376498E-061, 6.83970487E-063, -4.85373315E-064, 2.37322320E-066, 2.86446058E-066, -5.33943275E-068, 4.20511047E-069, 1.41594706E-072, 3.20192157E-071, -4.28311096E-072, 5.24662112E-073, -5.37580917E-073, 1.33098773E-073, -7.51416400E-074, -3.41872393E-075, -3.65693099E-075, 4.51928811E-076, -3.25481864E-076, -1.99622750E-076, 4.05604674E-077, -9.69186629E-079, 4.17705098E-078, 1.06027967E-078, 9.91620459E-079, 2.06988222E-079, -5.85035372E-080, -8.22027584E-080, -2.59848549E-080, -5.32656225E-081, 6.04355642E-083, -1.70490298E-081, 2.03588988E-081, -6.22363694E-082, -7.47176857E-082, -4.90104453E-083, 4.24033008E-083, -1.61153673E-083, -3.04341744E-083, -4.42197335E-084, 4.57991577E-084, -1.98906382E-085, 1.43684751E-084, 6.83036846E-085, -8.65937221E-085, -1.87998248E-085, 2.30691901E-085},
        {-3.76739603E-008, -1.52221829E-009, 2.16396230E-011, 9.41294467E-013, -8.35744038E-015, 4.70310881E-016, 8.68708866E-018, 1.29577390E-020, -3.31151671E-021, -2.24126621E-023, 4.67107708E-024, 1.19644257E-025, 2.67335728E-027, -2.81850805E-029, 1.56006162E-030, -1.20992853E-032, -1.59994785E-034, -1.36940695E-035, -3.97288626E-037, -1.36332607E-038, -2.87752803E-040, -4.85614932E-042, -1.09210646E-042, 2.62155023E-045, 5.21857749E-046, 8.93816528E-049, 5.82357088E-049, 3.96443534E-051, -1.28317243E-052, -1.40256126E-053, 1.96308797E-055, 3.36734960E-056, -3.10257736E-058, -1.50294312E-059, -5.30522580E-061, -1.09481036E-061, 1.34742011E-063, 3.39022411E-065, -1.11726620E-065, -3.56807349E-067, 2.25803510E-068, 2.55905008E-069, 2.18994583E-071, 3.36239841E-072, -9.07714670E-074, -2.72166448E-073, 3.21489011E-074, -6.49248050E-075, 8.72478506E-076, -4.15869700E-077, -1.04829051E-076, 2.71862302E-077, -1.67387331E-077, 2.46226935E-078, 2.02570395E-078, -3.35218020E-079, -7.65086156E-081, 2.97868065E-080, -1.65022657E-080, -5.75304676E-081, 3.94084647E-081, 1.74648515E-081, -9.12514932E-082, 3.36716789E-083, -4.04368423E-083, -9.56323766E-083, -3.07317384E-083, 1.20756672E-083, 5.88024701E-084, 2.68141389E-084, 1.05682546E-084, -1.34762941E-085, 1.72875693E-085, 1.45059067E-085, 9.13836928E-086, 4.32034707E-086, -1.27668303E-086, -4.15064944E-087, -4.81743363E-087, 2.02335840E-087, -3.48912212E-087},
        {-1.22947043E-008, -8.93851465E-010, -1.63079582E-011, 1.70984469E-013, 2.20802483E-014, -8.69429138E-016, 5.68566830E-018, 2.32769767E-019, 2.97343520E-021, 1.79053981E-023, 3.99060484E-024, -1.46537645E-026, 2.29310456E-027, 7.57480395E-030, -1.84520557E-030, -7.79012969E-033, 4.63417832E-034, -7.84325295E-036, -1.47986950E-036, -8.26513814E-039, 4.42201069E-040, 5.05480990E-042, 8.64271587E-044, -5.19127851E-045, 2.15643266E-046, -5.47810791E-048, -6.74687432E-050, 9.18717289E-051, -1.23906881E-052, -7.28501824E-054, 1.08002666E-055, 6.44102840E-057, 3.02711290E-058, 8.96332960E-060, 1.50212018E-061, -1.31794163E-062, 4.60628252E-064, -2.81996665E-065, 6.25260229E-067, 5.71891982E-068, 1.25226923E-069, 9.59717265E-071, -5.14155043E-071, -7.59890909E-073, -2.76256105E-074, 3.38586905E-075, 3.05811223E-075, 1.51470958E-076, 7.66149988E-077, -2.32870352E-077, -2.24912608E-078, 5.68293173E-079, -2.17042522E-079, 1.25225019E-079, 3.65752949E-080, 2.82415544E-081, -5.70379535E-081, -3.22712063E-081, -4.24165111E-082, -1.78310132E-082, 2.34780202E-083, 5.39948238E-083, -3.33824694E-084, 6.74015701E-084, 1.49888028E-084, 1.19525332E-085, -4.27482463E-085, -2.37459275E-085, -7.22033077E-086, 1.07916762E-086, -1.98233171E-086, 6.35866658E-087, -1.85185233E-087, 1.88920437E-087, 3.72365470E-087, -2.72568715E-088, -5.79460744E-088, -2.00922782E-088, -1.46845435E-088, -2.98433167E-089, 8.59231448E-090},
        {4.47729640E-008, -9.07241028E-010, -6.94448993E-011, 5.09048701E-013, 4.40600521E-015, -7.53918699E-016, 1.18635554E-017, -7.02259417E-020, 6.00227015E-023, -6.11204575E-023, 1.71001904E-025, -4.83783742E-026, -8.30073308E-028, 1.58694481E-029, -1.10437494E-030, 1.68539530E-032, 3.19456938E-035, 1.14485263E-035, -2.67126307E-037, -8.05998607E-039, -3.31408822E-041, 4.84180925E-042, 2.27444374E-043, 4.11916578E-046, 1.53845917E-046, 2.82836343E-049, -6.65187771E-050, 2.39750353E-051, -1.56651589E-053, -5.23336319E-055, -1.61287949E-055, -2.27741634E-057, -6.09164812E-059, 3.50109326E-060, 6.23456389E-062, -1.01384888E-063, 2.04418396E-065, 4.19863493E-066, -3.49224022E-067, 1.47106683E-068, 1.88669874E-069, -3.24613220E-071, -5.84741844E-072, -1.76676305E-073, -1.58353135E-075, 3.98304841E-076, -5.10872244E-077, 3.03001405E-077, -7.39480522E-080, 1.16162774E-078, -3.82403788E-080, 1.79182328E-081, -5.00939031E-082, -1.73408947E-082, 1.06902495E-081, -2.35304547E-082, 7.96057215E-083, 7.50039918E-084, 8.65188663E-085, -6.18105187E-084, -5.66893492E-085, 1.50737881E-084, 2.22772076E-085, -1.24733050E-085, 4.72039865E-086, 1.91398684E-086, -1.81177137E-086, 9.92648595E-088, 3.51240914E-087, -8.66869666E-088, 2.41048838E-089, -5.30848917E-089, -1.48008350E-088, 4.44687960E-089, 3.60730986E-089, 3.29167083E-090, -2.60897770E-090, 3.86291287E-090, 2.04132603E-090, -1.46435088E-090, -2.79722886E-091},
        {6.76036514E-009, 1.30534552E-009, -2.55588782E-012, -2.95951665E-014, 9.02345739E-015, 9.46302702E-017, -1.43756583E-017, 1.18771305E-019, -2.40605632E-021, -8.32250783E-024, -2.51870857E-024, -2.39432733E-026, -1.13953725E-027, 1.52442252E-029, -7.66426638E-031, 7.42043771E-033, 3.68745051E-034, 8.62826728E-036, 2.71236556E-037, -7.94727223E-040, -1.34576450E-041, -5.51053200E-042, 1.32834748E-043, 5.07094776E-046, 3.98968237E-048, 1.03286358E-048, -4.36853466E-050, 6.86338933E-052, -7.06311015E-054, -1.59475650E-054, 3.28063400E-056, -1.34136387E-058, -2.84843325E-059, -8.31070687E-061, -2.52698015E-062, -1.99433881E-063, 3.53958988E-065, 3.84630536E-066, 7.39742314E-068, 5.02334721E-069, -1.29331166E-070, 5.95648042E-072, -9.75521227E-074, 2.42426701E-074, 2.96741873E-075, -2.06161094E-078, 6.64870905E-078, 4.02471007E-079, 1.50203214E-079, -1.02221134E-080, 7.84574940E-081, -2.62379284E-081, -1.06624569E-082, 8.07698448E-083, -5.14782264E-083, 1.25440110E-083, -7.49718342E-085, 1.33365448E-084, 5.70917454E-085, -7.50495270E-086, -6.70197826E-086, 1.66624963E-086, -8.46742901E-087, -3.25326269E-087, -6.56120353E-088, 3.21808020E-088, -5.42771168E-089, 7.23963290E-089, 3.43571023E-089, 5.01583694E-090, 1.14001493E-089, -1.31465953E-090, 5.57020703E-091, -5.99112283E-091, -7.91565162E-092, -2.89915937E-091, -1.33681365E-091, 1.49815772E-092, 2.84534991E-092, 2.72905424E-093, -1.02992317E-093},
        {-4.01190764E-008, 1.31591678E-009, 7.72396299E-012, -4.40648952E-014, -1.31950620E-015, 2.18884085E-016, -8.63454378E-018, 4.98023968E-023, -2.71734364E-021, 1.16992317E-022, -5.28210036E-025, 9.98179388E-027, -9.52007224E-028, -2.21896737E-029, 2.27990636E-031, -7.58474491E-033, 3.73398266E-034, 3.31434376E-036, 7.76073529E-038, -3.90274722E-039, 1.02878955E-040, -1.58003832E-042, 3.61812935E-044, 6.39086471E-046, -8.13710357E-047, 1.57394024E-048, -1.11139526E-050, -8.79007129E-052, 2.11496964E-053, -8.11295925E-055, -5.51947825E-057, -1.66630932E-058, -6.07175637E-060, -1.47217148E-061, -7.00644385E-063, -6.96606808E-064, -2.80662230E-065, -7.33146224E-067, -2.20628363E-068, -4.67691993E-071, 3.63606810E-071, 4.89055666E-073, -3.36766996E-074, 6.01266990E-075, 1.22740752E-075, -2.34210830E-077, -3.62667431E-079, -5.54883157E-080, 1.65370010E-080, 4.47852597E-082, 1.28169431E-082, -3.94608516E-084, 1.35501246E-083, -5.80085113E-084, -1.39807445E-084, -1.99511922E-085, -1.35674601E-085, 2.52587640E-086, 7.55713732E-087, 1.80549474E-087, 1.49662815E-088, 2.81803250E-088, 4.60053964E-089, -2.48447120E-089, 4.82048437E-090, 7.60827382E-090, -5.82616207E-090, -4.66787866E-092, -1.05783313E-091, -2.59338406E-091, -9.75108144E-092, 7.60367706E-092, 9.29349591E-093, -1.50216967E-092, -2.25966190E-093, 4.75914854E-093, -2.01058891E-094, -1.00857151E-093, 2.66660955E-094, 8.88537254E-095, 2.44726766E-095},
        {-2.60390543E-008, 5.28870970E-010, 3.47018838E-011, -2.95251337E-013, 1.66000866E-015, -2.48418782E-016, -7.12989643E-018, 1.63637582E-019, 1.40891133E-021, 1.41473354E-022, 6.57231669E-025, -2.52963921E-026, -1.51164946E-028, -1.86838601E-029, -3.15753227E-032, -9.80238077E-033, 1.15118568E-034, -3.39556096E-036, 5.38271688E-038, -2.52392199E-040, -3.82744564E-041, -1.53094385E-042, 4.10102011E-044, 4.18760956E-046, -1.95471809E-047, 4.35314844E-049, 1.06140465E-050, -1.64671924E-052, -9.72396727E-055, -1.14063784E-055, -7.55367928E-057, -9.91153181E-059, -2.03516288E-060, 5.39509390E-061, -1.98517739E-063, -2.30070111E-064, -3.13128315E-067, -1.54246503E-067, -9.38042384E-069, 4.55815851E-070, 2.76965063E-072, -3.81314234E-074, 4.70610127E-075, -6.26234589E-076, 3.53773995E-077, -2.01488319E-078, -5.12501601E-080, -6.62839892E-081, 6.20734598E-082, 8.90400403E-084, -3.99885769E-083, -5.50485563E-085, 8.26517333E-085, 2.26682830E-085, 3.30321873E-086, 1.04175903E-087, 2.39943219E-087, -4.24907074E-088, 1.32574931E-091, 1.72885861E-088, 2.19385426E-089, 1.51759726E-089, 3.55216233E-090, -6.14316209E-091, 7.69855063E-092, 2.24544260E-091, -4.91611504E-092, -4.16955994E-092, -8.51460772E-093, 5.02452538E-093, -3.31026403E-093, -1.05188824E-094, 4.67645493E-096, -3.08080053E-094, 7.24380748E-095, -5.33051158E-095, -4.20319006E-095, 8.13859892E-096, 1.09911738E-096, 4.92684791E-097, -7.85728428E-097},
        {-1.85339397E-009, -1.82027269E-009, 2.45540429E-011, 3.34250399E-013, -2.84124250E-015, -2.48152718E-017, 3.13868079E-018, 9.14123029E-021, 2.96980081E-021, -1.53675015E-023, 1.62769884E-024, 6.77322657E-027, 1.24194558E-027, -7.03640914E-030, 6.13441211E-032, -2.33403282E-033, -5.73007945E-035, -1.16651854E-036, -2.50706083E-038, 8.41511739E-040, -8.38533101E-041, -1.05389154E-042, -3.13998865E-044, 3.07199008E-046, -2.81863226E-048, -1.56227507E-049, 1.23743744E-050, -2.28223315E-052, 1.61244742E-054, 1.49010248E-055, -1.65145467E-057, 9.73851436E-060, -8.43300988E-061, -7.09340675E-062, 8.42260289E-064, -1.25811069E-064, 3.24803421E-066, 1.18670999E-067, 3.20491543E-071, -3.29770952E-071, -5.73045878E-072, -1.00194165E-073, -1.86907533E-075, 1.24000345E-076, -9.91095800E-078, 1.08548680E-078, -3.31425285E-081, 4.34735115E-081, -2.04209604E-083, 7.60099731E-085, -6.26724183E-086, 2.65739168E-086, 2.49139108E-086, -5.34400190E-087, 7.02953210E-088, -2.18948733E-088, -9.35264078E-089, 4.18398301E-089, -1.00047205E-089, 5.41255161E-090, -1.10235503E-090, 2.56320340E-092, 8.40389736E-092, -2.28047283E-092, -1.41929723E-092, -1.25559338E-093, 7.13523031E-094, -9.80425428E-094, -7.00286056E-095, 5.48982788E-095, -7.72760432E-095, 3.43651954E-095, 2.46514685E-095, -5.13074296E-097, -9.51872930E-097, -1.15456117E-096, 2.95439513E-097, -2.16211138E-097, 5.71278641E-100, -6.93931426E-099, 2.49538000E-098},
        {3.52275106E-008, 3.30188410E-010, 2.67805657E-011, -1.50360450E-013, 1.03505304E-015, 2.84670334E-016, 5.16044089E-018, -9.02038079E-021, 1.20716366E-021, 2.21474083E-024, -7.33744271E-025, 6.06319829E-027, 8.56587678E-030, 5.65280261E-030, -1.78075135E-032, 4.26028319E-033, 1.71554573E-035, 4.56866137E-037, -6.33768216E-039, -4.93219584E-040, -2.56827847E-041, 4.66591617E-044, -1.90843328E-044, -1.23052338E-046, -6.00738481E-048, -1.05114904E-049, 5.48297980E-052, -1.62753856E-052, 3.09758614E-055, 2.65533055E-056, -4.00778128E-058, 1.92271127E-060, 4.17933606E-061, 1.34311544E-062, 1.38890013E-064, -3.02123184E-065, -7.66610930E-067, -9.32555943E-069, -1.48908045E-069, 2.96028623E-071, 2.94451612E-073, -1.81696670E-074, 5.88324708E-076, 6.38967165E-077, 4.00044334E-079, 1.78548142E-079, -6.96879857E-081, 4.37383206E-082, 8.52594253E-083, 1.67415918E-085, -6.76474760E-086, -5.32248236E-087, 2.16913123E-087, -2.81871057E-088, 6.01055114E-090, -7.44509051E-090, 6.28734178E-090, -8.41390605E-091, -4.71690767E-092, 8.69561037E-092, -3.65120907E-092, -2.88123409E-093, -1.09583776E-093, 7.33524136E-094, 2.70755694E-094, 7.51849814E-095, 5.60861696E-095, 1.77903671E-095, 6.40352932E-096, 1.68904575E-096, -2.81473405E-097, 1.85209304E-098, -1.07760554E-097, -7.23394241E-098, -6.75599176E-099, 4.19536648E-099, 8.57972708E-101, 2.88402095E-099, 2.16355287E-099, 7.32929827E-100, 1.76710658E-100},
        {-1.31717682E-009, 1.83949945E-009, 7.48094146E-013, 1.67068940E-013, -9.23479875E-016, 1.21967534E-016, -2.05174137E-018, 6.23738686E-020, -4.31258396E-022, -6.76063193E-024, -8.47595705E-025, 1.70768039E-026, -2.34723378E-028, 3.82178753E-030, 2.03778676E-032, -1.35205104E-033, -1.78074438E-035, -8.22001181E-037, -8.77953535E-039, -3.34250938E-040, 1.49329556E-041, -1.48362669E-043, 3.04321762E-046, 1.08287676E-046, 3.61392056E-048, -8.86780367E-050, -3.56498866E-051, -3.30621659E-053, -8.52654175E-055, 4.33138804E-057, 4.07131623E-058, -1.04663689E-060, 1.51372117E-061, 4.45996911E-063, 3.63421264E-064, 6.48441763E-066, -3.13224311E-067, -1.98599184E-069, 1.90010347E-070, 5.56668935E-072, -1.16702749E-073, -4.14631068E-075, -2.48114285E-076, 1.70815227E-077, 8.50565278E-079, 5.80617648E-081, 1.12664837E-081, 4.38721484E-083, -4.59010558E-084, 5.12023751E-085, 2.17779032E-087, 9.33850842E-088, -2.92277855E-089, 5.91968232E-090, -5.39766711E-091, 1.39155285E-091, 4.14981323E-092, -3.95777619E-092, 1.82991099E-092, 2.05677827E-093, 9.04589182E-094, 2.10463861E-094, 5.29438507E-095, 1.17195114E-095, -1.02944397E-095, 1.14640249E-096, 4.94057939E-097, -2.10129717E-097, 1.02702957E-098, -1.87948292E-098, 3.24013052E-099, -3.36921761E-099, 1.42176031E-099, 2.23742856E-099, -1.46120474E-099, -5.56975607E-100, 6.72612958E-104, -9.42212296E-102, 2.07131725E-101, 2.16594936E-101, 6.14384745E-102},
        {-3.78003113E-008, 1.19304556E-009, -4.80434400E-011, 7.23878784E-014, -1.86271790E-014, -3.31094349E-017, -7.00402698E-020, 3.71674969E-020, -9.47012121E-022, -1.00873229E-023, -6.89333676E-025, -8.39744184E-027, -2.34883669E-028, -7.52576229E-031, -9.24291614E-032, -1.71391892E-033, -2.22506720E-035, 3.71536177E-038, 2.94845012E-039, 6.33882398E-041, 4.41094340E-042, -2.87580300E-044, 2.62502487E-045, -3.71228315E-047, 5.17417826E-048, 5.14984195E-050, -2.18818952E-051, 3.28460988E-053, -1.05435503E-055, 2.05835412E-056, 3.57954075E-058, -5.85979880E-060, -8.47580022E-062, -6.21456823E-063, -3.02340158E-065, 1.15367444E-066, 5.75969515E-070, -5.66896754E-070, -2.33902661E-071, -3.70089209E-072, 6.38567399E-074, -4.68207640E-075, 1.45848219E-076, -4.93152177E-079, 2.96512743E-081, -4.82419941E-081, -2.97402937E-082, -1.98901606E-083, 1.97351385E-085, 3.34046626E-086, 8.00119191E-087, 5.19043834E-089, -3.64020427E-090, 1.27250427E-091, 1.31617201E-091, 3.21552391E-092, -1.66482374E-093, -1.25266346E-093, -2.04160865E-094, -1.22871899E-095, -4.09545873E-096, 5.12091723E-096, -2.65371008E-096, 8.97595958E-098, 3.21784229E-098, -7.34804267E-098, -2.08496735E-098, -8.37776166E-099, -2.58284266E-099, -7.81872707E-100, 5.35343605E-100, -7.38000684E-101, 6.62987360E-101, -3.41010925E-102, -1.91173467E-101, 2.23910361E-102, 8.99905973E-103, -1.60640187E-102, 2.80869017E-104, 1.98503554E-103, 1.10083907E-103},
        {-4.85982622E-008, 4.61116518E-010, -3.74200651E-011, -4.68446753E-013, 2.20950952E-015, -1.19499072E-016, 7.56132327E-019, -2.03964606E-020, 4.19623004E-022, -5.78334582E-024, 4.40949238E-025, -8.64466815E-027, -2.05370221E-028, -9.27442612E-030, 2.96033147E-032, 3.26171720E-034, -1.18742948E-035, 5.76818618E-037, 4.74666720E-039, 5.24975576E-041, 7.72873612E-043, 4.69630551E-044, -1.87158684E-045, -3.12372653E-047, 1.08242458E-048, 4.50825623E-050, -1.92730435E-052, 1.62441885E-053, 2.76554046E-055, -4.35609255E-057, 7.35248444E-059, 3.79398489E-062, 6.17017434E-062, -1.64615996E-063, -7.18874091E-065, 1.55738476E-066, -7.12106699E-068, -7.54257349E-070, 1.70796511E-071, 1.33585414E-072, -2.92830902E-074, 3.91629583E-076, 8.72467724E-079, -1.30013745E-078, 5.85519513E-082, -3.58565591E-081, 7.10576380E-084, 2.88845267E-084, 1.39221255E-085, -3.81641647E-087, -2.55225152E-088, -8.53187611E-090, -6.15051081E-091, 1.10931900E-093, -4.19562263E-093, -9.92585175E-094, -1.28216299E-094, -8.20755122E-096, 1.72601227E-095, -7.10855206E-097, 1.19346206E-096, -7.18768786E-098, -9.62537847E-098, 3.11817667E-098, -9.04783778E-099, 1.12315244E-099, -1.67842653E-100, 9.41329157E-101, 2.14276650E-101, 6.13125594E-102, 1.56990910E-101, -1.31145095E-102, -6.64612744E-103, 5.05502152E-103, 2.62764614E-103, -8.78897360E-105, -2.38724143E-104, 4.43063516E-105, 1.96845142E-105, -3.41256895E-105, -4.62999435E-106},
        {8.98451019E-009, -3.81346156E-010, 1.83240618E-011, 1.04958055E-013, 6.83367969E-015, -5.99790626E-018, -3.94910844E-018, -5.56405345E-020, 2.79966364E-022, -1.12644418E-023, 4.76060987E-026, 1.59930757E-027, -2.54191280E-029, -2.02088306E-030, 4.22079016E-032, 8.63803568E-034, 6.04458147E-036, 6.32781879E-038, 1.53733979E-040, -8.01702633E-042, 2.39041389E-042, 1.84197877E-044, -1.18809482E-045, 4.15360168E-047, -3.84872960E-050, 1.08350791E-050, -5.35251827E-052, 3.62386902E-055, 9.98485766E-056, -5.95268339E-057, -4.36314968E-059, 1.04522287E-060, 5.07804785E-062, 2.17741728E-064, 3.98445600E-065, 1.36847654E-066, 5.73551133E-069, -1.22356517E-069, -3.92239262E-072, -1.64891188E-073, -3.22018387E-074, -2.50172309E-076, -2.09247323E-078, 4.39439044E-079, -3.01848886E-081, 2.41471615E-082, 1.32447542E-084, -5.62248944E-085, 1.95188782E-086, -2.76030373E-087, -8.98615087E-089, -7.88288615E-090, -4.80854942E-091, 7.28837095E-093, 3.97434765E-094, 3.79522006E-095, -1.08809211E-096, -1.09983686E-096, -3.15240208E-097, -1.01418364E-097, 4.19989590E-098, -8.82023035E-099, -1.56489945E-099, -1.68648420E-101, 3.92612196E-101, -1.09501793E-101, -2.61625939E-101, 4.65848489E-103, 6.57466352E-104, -1.00991287E-102, 3.99122828E-104, -4.55464563E-104, -2.35698514E-104, -1.07257261E-104, -7.57223581E-106, 1.40176153E-105, 7.11568792E-106, 7.30050476E-106, -9.30636550E-107, -8.40915372E-107, -1.56789009E-107},
        {7.64856307E-008, -8.33229529E-011, 3.72666603E-011, -2.03218461E-013, 6.42895270E-015, -2.28653940E-018, 5.96549419E-019, 1.59484626E-020, 4.72236237E-022, 6.04105466E-024, 9.03980596E-025, 3.30088999E-027, -1.09450079E-029, 2.32295719E-030, 3.62364159E-032, 1.21624582E-033, -3.12196851E-036, -3.04382913E-037, 6.49819291E-039, 1.30086115E-040, -1.42721527E-042, -2.88750030E-044, 8.24019830E-047, 2.18024064E-047, -6.57668731E-049, 1.46757398E-051, -8.51038575E-053, 3.49568255E-055, 2.31066756E-056, -3.62233079E-057, -6.22111128E-059, -8.42193964E-061, 1.64793258E-062, -1.44084550E-064, -1.75372829E-065, 1.09693128E-067, 7.19056449E-069, -1.30040355E-070, 2.60765922E-072, -8.63612055E-074, -3.79644699E-076, 2.26728237E-076, 4.59862687E-078, -2.43268951E-080, 1.69411828E-081, -2.16934858E-082, -2.02476724E-085, -2.19336189E-086, -5.34700020E-087, -8.72450217E-089, 2.96556262E-089, 5.43773917E-091, 8.36360212E-092, 4.18838141E-093, 7.25176374E-095, -7.08005212E-096, -2.85332172E-097, 7.97495788E-098, -3.29047738E-099, -2.52807646E-099, -7.09309191E-100, 3.19433590E-101, 7.44552478E-101, -1.91359203E-101, -2.22528919E-102, 6.75876323E-103, -3.79619970E-103, -3.06954450E-104, -1.23588192E-105, -1.76401549E-104, 4.63783529E-105, 4.38417714E-106, -3.98455335E-106, 1.78341096E-106, 2.93165351E-106, 2.18597960E-107, -1.39819085E-107, 7.36729936E-108, -1.24318990E-108, -1.05653852E-108, -3.18048697E-109},
        {2.44768219E-008, -4.42491246E-010, 3.08855810E-012, 5.64190081E-013, -2.58233173E-015, 5.28845552E-017, 1.54954609E-018, 2.62822825E-020, 6.04332910E-022, 1.43090906E-023, 1.18759831E-025, 6.15559862E-027, -1.57280297E-029, 1.64350704E-030, -2.96366532E-032, 5.74448348E-034, -8.12544254E-036, -4.95242330E-038, 2.63756700E-039, 1.82880612E-040, -8.86375708E-043, -3.10144501E-044, -4.20499267E-046, 9.54938034E-049, 1.89134144E-049, -4.87436086E-051, 1.26916875E-052, -5.66403982E-056, -2.16832190E-056, -7.58946656E-058, 1.30953325E-059, 6.68503478E-063, -6.46723061E-063, -4.28269388E-064, -1.39529051E-065, 2.44445721E-068, 2.09022772E-069, 4.93461680E-071, -3.31973594E-073, 1.24743663E-073, 1.07420271E-075, 6.86942600E-077, 1.97648296E-078, -1.90849862E-080, -6.58742363E-082, -1.58660198E-083, 4.17201012E-085, -2.46662486E-086, 1.49605275E-087, -1.91688406E-090, -1.37926408E-090, 1.67617503E-091, 2.92777608E-093, -5.50411687E-094, -9.69181912E-095, -8.35953431E-097, -3.52955354E-098, 1.34140383E-099, -7.75769944E-100, 9.11083311E-101, 2.88328961E-101, -2.97782417E-102, 1.62936207E-104, 1.78307858E-104, 1.03088034E-103, 5.10267589E-104, -9.40898267E-105, 9.67700052E-105, -1.11581874E-105, -2.65802683E-106, 1.00424369E-106, 1.67704994E-108, 2.28454035E-108, -1.93848545E-108, -5.66548164E-109, -4.34767120E-109, -2.68716302E-110, 2.18475712E-109, 3.58380030E-110, -3.31293547E-111, 2.98753680E-111},
        {1.17473126E-008, -4.31899999E-010, -1.00496724E-011, 4.17513179E-013, -4.42112070E-016, 1.82739422E-016, -1.55478244E-018, 1.94168346E-020, -6.38503174E-022, 8.54084558E-024, -1.82592269E-026, 2.03803716E-027, 2.29143364E-029, 4.44543613E-031, -6.22865349E-033, -2.29514543E-034, -9.55323779E-036, -7.28346607E-038, -9.70450170E-040, 1.83095767E-041, -3.31854223E-043, 4.49859825E-045, 4.23498951E-046, 3.92701470E-048, 2.92511954E-049, -4.35852174E-051, -1.68461629E-053, -1.54636291E-054, 5.73464953E-057, 3.85479895E-058, 1.70052521E-059, -5.23548495E-063, -8.91432133E-064, 8.92506329E-065, -1.64910978E-066, 5.43661691E-068, 8.55327569E-070, -3.36958935E-071, -1.97529763E-072, -6.97300609E-075, 3.41206832E-076, 1.73983729E-077, 3.30669180E-079, 3.04948464E-081, 7.80344290E-082, 3.48993965E-084, -2.83566898E-085, 1.04108051E-086, -5.09076198E-088, 1.09928157E-089, -5.35289065E-091, -2.20349831E-093, -1.33471115E-093, 1.33094586E-095, -6.94986176E-096, 2.47238365E-097, -1.55388101E-099, 2.47439953E-100, -2.25285231E-102, -5.55729769E-102, -1.46540439E-102, 2.58116768E-103, -8.31289519E-105, -2.74131059E-104, 4.59057016E-105, -1.20434528E-105, 5.31869419E-107, -1.33790215E-106, -5.22390367E-107, -4.70305295E-108, 2.53209732E-108, -5.80676685E-110, -3.17909579E-109, -1.39632167E-109, 2.46087401E-110, -5.45481562E-111, -3.80593614E-111, 2.78630319E-111, -1.44431124E-112, -8.96962851E-113, -4.04656962E-113},
        {-5.02103658E-008, 1.04621875E-009, -2.31411938E-011, -9.37373419E-014, -2.82640388E-015, 6.28418396E-017, -3.72864837E-018, 1.76879362E-020, -7.02239920E-022, 1.11117436E-023, -2.11079679E-025, 2.57056316E-028, 6.57148847E-030, 1.02558261E-031, -6.18034509E-033, -2.29291732E-034, 3.06067491E-036, -2.65651951E-037, -1.77352894E-039, -1.63141387E-041, 1.04665986E-042, 1.74065376E-045, 7.89880268E-047, 3.23082920E-048, 4.28548153E-050, -1.14926156E-051, 2.64010152E-053, 1.73259462E-055, -1.58542563E-056, 4.28509825E-058, -1.72735584E-060, 2.54148664E-063, 2.92789816E-063, 1.12925801E-064, 9.87008042E-067, -1.16909782E-068, 7.07461983E-070, 1.29191940E-071, 1.54752205E-073, -3.12425191E-075, 1.80070327E-076, -1.47483103E-078, -2.09052502E-079, -8.70246574E-082, 4.99873192E-083, 4.26571867E-084, 1.11872841E-085, 6.91737893E-087, -5.75751078E-089, 5.27358667E-090, -1.59826589E-091, 3.87883248E-093, -2.54500364E-094, -2.90250622E-095, -1.17213859E-096, 9.04497887E-100, 1.57696564E-099, -3.11390500E-101, -4.50998067E-104, -2.74310626E-103, -6.16750362E-105, 1.13135898E-104, -3.38831842E-105, 2.44211879E-106, -1.85176468E-107, 2.90453324E-108, 1.21108837E-107, -1.87930661E-108, 2.06768519E-109, -8.46973589E-110, -3.60580030E-110, 1.54203769E-110, 8.86241903E-111, 5.95839204E-112, 5.21777041E-112, 1.49723081E-112, 4.43862666E-113, 8.89921136E-114, -6.77395341E-114, 1.41001377E-114, 1.58332832E-114},
        {-3.44416664E-008, 8.47512233E-010, -1.23603511E-011, -4.54027606E-013, -1.05864091E-014, -7.88355866E-017, 2.80692325E-020, -3.33371666E-021, 5.81423748E-022, 1.14321375E-024, -5.59433845E-026, -3.71128711E-027, 5.54587473E-029, 3.27075723E-031, -2.33618363E-033, 2.86914911E-035, -1.17516363E-036, -2.67282980E-038, 6.46780630E-040, -3.41228518E-041, 4.39584547E-043, -7.85552506E-046, 1.40481363E-046, -2.65306088E-048, -3.19844672E-050, 2.47077420E-051, -1.46414975E-053, 9.09030981E-056, -1.05858522E-056, 1.65834935E-059, 6.96181800E-060, -1.75069974E-062, 1.46702042E-064, 9.92818690E-066, -4.97811185E-067, -1.76493069E-069, -7.82460175E-071, 5.35134989E-072, 2.45590953E-073, 2.38560235E-075, 4.75947076E-077, -8.83157571E-079, -1.43927878E-079, -9.02567029E-082, 1.83105348E-083, -4.33455024E-085, -4.20638465E-086, 1.55575010E-088, -1.07334560E-089, 1.69012491E-091, -1.22664486E-092, -1.02427492E-093, 3.90292370E-095, 1.92707172E-096, -1.73890482E-097, 4.10774452E-099, 8.29981502E-101, -2.73853560E-101, 3.33296347E-104, -1.88725300E-104, 2.35437590E-105, 1.04285758E-105, -4.60053556E-107, -6.92610360E-108, -5.33486037E-109, -3.12939170E-109, -1.01105260E-109, -4.04867505E-110, 5.12538155E-111, 4.21692406E-111, -1.70731989E-111, 6.23441231E-112, -1.46385830E-112, -1.15507772E-113, 1.92617953E-113, -3.55169324E-114, -2.10540929E-114, -7.27358121E-115, -4.20164919E-115, -1.86234381E-115, 1.99994124E-117},
        {-3.28854580E-008, -2.83668091E-010, 4.12805284E-012, -1.58099245E-013, -4.29550869E-015, 4.77714749E-017, 3.21617094E-019, -4.43074049E-020, 3.76458273E-022, -9.11036572E-024, 9.97043669E-027, -1.55773768E-027, 1.84778502E-029, -7.19380187E-031, 1.11214272E-032, 6.55093975E-035, 2.56577828E-036, 7.15028880E-038, 8.97658556E-040, 9.75468659E-042, -2.61132764E-043, -1.81808175E-045, 1.25018340E-046, -1.49477781E-049, -4.56081531E-050, 5.81051178E-052, -5.50772382E-054, -2.67236705E-055, -4.14923287E-057, 9.32253031E-059, 3.74949656E-060, -4.26442796E-062, 5.98508520E-064, -3.89951447E-066, -9.34882186E-068, -6.33451025E-069, -1.63559581E-070, 3.36155702E-072, -1.28531006E-073, 3.52311334E-076, 1.89677959E-077, 3.43706516E-079, 6.95539618E-081, -9.74951300E-083, 2.83656581E-083, -6.31795532E-086, 4.01731919E-087, 5.83256767E-089, -4.52082163E-090, 1.55372038E-091, -1.98198691E-094, 1.03835754E-094, -1.56994740E-096, -6.15204828E-097, -1.34040083E-098, 1.90400307E-100, -1.50385215E-101, -2.04293446E-102, -4.68816058E-105, 4.95838255E-106, 7.94504335E-107, -1.81077365E-107, -3.38108941E-108, -1.02744953E-108, -1.96714209E-110, -2.48464621E-110, 9.60843557E-111, -2.07820394E-111, 7.70711391E-112, -2.56299596E-112, -3.66842352E-113, 1.20970191E-113, -1.63691256E-114, 3.31581105E-115, -7.01038452E-115, 2.30778618E-116, -3.08680728E-116, -1.61236183E-116, -1.77420834E-117, -3.64554369E-118, 3.67422784E-118},
        {2.00725778E-008, -1.11684416E-009, 2.59710550E-011, -2.04899575E-013, 4.22969922E-015, -1.27353079E-017, -2.00965110E-018, -1.87702257E-020, 3.93099277E-022, -3.32526788E-024, 1.56636010E-025, 1.84664062E-028, 2.14524357E-029, -7.16541582E-031, 1.12151748E-033, 2.85044385E-037, -1.80323292E-036, 3.92062108E-038, 8.34329005E-040, -9.32599919E-043, -2.99494405E-043, 5.77083994E-045, 2.54544306E-047, 1.96732609E-048, 4.31803802E-051, 1.86145905E-052, 1.97877358E-054, -1.12172634E-055, 6.19702683E-058, -6.85453171E-059, -9.58872780E-061, -1.45093141E-062, 1.68536288E-064, -2.17890104E-066, 1.37393750E-067, -1.65266181E-069, 4.93837581E-072, -4.79854284E-073, -1.56597239E-074, -4.82874014E-076, -4.69869362E-077, -2.22862099E-079, -6.69591042E-081, -1.82056135E-082, 1.01385832E-083, 2.96974072E-085, 5.51372636E-087, -6.15911871E-090, 2.65752226E-090, -5.28533925E-092, -3.18584788E-093, -9.42127263E-096, -4.90135204E-096, -1.14040515E-098, -3.34822533E-099, -3.29390082E-101, 8.07715682E-102, 4.08602519E-103, 6.32492371E-105, 1.78876119E-105, 2.36366207E-108, -1.78482713E-108, 9.52092434E-110, -1.23440145E-110, 2.62137239E-111, 8.58724387E-112, -2.83060321E-112, 1.15511291E-113, 9.44732791E-114, -1.84908976E-114, 1.45708842E-114, 1.20796607E-115, 1.60036764E-115, 2.13581966E-116, 4.67048807E-117, 1.61273681E-117, -4.60982353E-119, 1.09178674E-118, 4.92319641E-120, -3.96272081E-119, -5.36667647E-120},
        {-4.80818064E-009, 3.47234559E-010, 1.50567520E-011, 2.06957638E-013, 8.31338636E-015, -4.14296321E-017, -1.38105424E-018, -9.86672770E-021, 2.28676772E-022, 9.21988284E-025, 1.07945625E-026, 1.10365218E-027, 6.44064353E-030, -6.08425020E-032, -4.95952522E-033, 3.74401180E-035, -1.95625006E-036, -4.98332918E-038, -4.01248212E-040, 1.07836416E-041, -5.07014435E-044, 1.04952116E-045, 5.20907359E-047, 1.88000354E-048, -9.96850828E-051, 1.32929921E-052, 6.35254851E-054, -1.36929978E-055, 8.22230156E-058, 2.25303148E-059, -9.16810120E-061, 1.57319704E-062, -5.03102813E-065, -7.13637030E-066, 1.37520467E-067, 8.61849123E-070, 1.07252894E-071, -7.39622659E-073, 7.06875992E-075, -2.58178776E-076, -1.50867659E-078, -4.41119238E-080, 2.17192859E-081, 1.30766248E-082, 1.00308513E-084, 7.08111350E-086, 5.09293672E-088, 8.63966627E-090, -1.32499505E-090, 4.85911880E-092, 8.38764142E-094, -2.08762106E-095, 7.40934138E-097, -6.58117875E-099, 1.24356643E-099, 2.37946850E-101, -6.02077786E-103, -3.33574342E-104, -1.46456967E-105, -1.52885639E-106, 2.54460870E-107, 4.28173589E-110, 1.02403423E-110, 2.44082687E-111, -2.23881713E-112, 4.49163119E-113, -6.72170787E-115, -2.03227917E-114, 8.73937234E-116, -1.95404211E-115, -2.19265607E-116, -7.17290264E-117, -2.80595633E-117, -1.98987451E-120, -1.81556193E-118, 1.06618371E-118, 2.77773655E-120, -2.71102846E-120, -2.39229395E-122, 6.54397707E-121, 7.01498058E-122},
        {3.23718857E-008, 3.24156348E-010, 6.07634467E-013, -2.48552949E-014, 2.06248126E-016, 1.01512611E-017, 1.11911553E-018, 3.15883508E-021, 1.29553408E-022, 2.98488596E-024, -1.05606227E-027, 7.49452336E-028, -4.60492201E-030, 4.11793655E-031, -1.82590656E-033, 6.04483495E-035, -1.17642962E-036, -8.44676222E-039, -4.96029152E-040, 2.52125327E-042, 1.35146249E-043, -2.14896121E-045, 1.82365910E-047, -1.05823971E-048, 1.34316364E-050, 4.54248401E-053, 1.15134117E-054, -3.08241156E-057, 2.19483123E-057, -6.14631237E-060, -1.04458990E-062, 6.47173270E-063, -3.09304178E-066, -2.77544382E-066, 2.47486673E-068, -7.20376665E-070, -8.74394192E-072, -4.81752703E-073, -1.14407693E-075, 1.86150991E-076, 4.12450805E-078, -6.72242751E-081, 2.13872870E-081, 3.39928376E-083, 2.55363237E-085, 1.40037529E-086, -1.25175540E-088, -1.71403314E-090, 1.54775537E-092, 3.19190354E-093, -1.01208435E-094, -8.22067987E-097, -2.66729467E-097, -4.84867916E-099, 1.88065961E-100, 1.22128678E-102, 2.82621631E-103, 1.82249120E-104, -3.24618512E-106, -3.78018575E-107, -1.56281351E-108, -8.13067423E-110, 7.28395224E-113, 8.19549971E-114, -4.37157406E-114, 3.76173033E-115, 1.44413709E-115, -1.36726635E-115, 8.23387959E-117, -3.45253846E-120, 7.02080893E-118, -1.34568023E-118, 2.20960404E-119, -7.79614276E-120, 1.09766225E-120, 2.47863224E-120, -8.09043651E-121, 1.77207727E-121, -3.61000763E-122, 1.36427828E-122, 2.73567036E-123},
        {2.90920808E-008, 2.93513359E-010, -2.03248306E-011, 3.57788034E-013, -3.29335471E-015, 6.25377218E-017, 4.70859070E-019, -7.92530125E-021, 4.12164518E-023, 2.94848710E-024, -3.38008063E-026, 4.69516547E-028, 1.80776294E-029, 2.54928708E-031, 2.45215542E-033, -3.12333070E-035, 7.26213402E-037, -7.14119589E-040, -3.52114883E-041, 3.40043013E-042, 7.25193901E-044, -1.00240884E-045, 1.29587448E-048, -4.80184619E-049, 9.68106342E-051, -8.39461103E-053, -1.59217590E-054, 6.68842528E-057, 1.84667085E-058, 4.98631146E-060, -3.43747296E-061, -3.33457855E-064, 1.44860762E-065, -1.55204961E-066, -5.56615168E-069, 8.09532048E-070, 6.50263890E-073, 2.06203654E-073, -2.47578875E-075, 8.75933943E-077, -1.06806889E-078, -3.93217089E-080, 7.42895438E-082, -7.93739149E-084, 2.53406711E-085, -2.24669539E-087, -1.84285414E-089, -2.42406832E-090, -4.54391749E-092, -4.09372371E-093, -4.02232482E-096, -2.93408994E-096, 7.43028240E-098, -5.61360626E-100, 5.75971492E-101, 2.33890214E-102, -6.18208282E-104, 7.76623706E-106, -4.20988512E-107, -7.51869082E-109, 7.44247927E-110, 1.85392988E-110, -8.18500212E-112, -5.77626506E-115, -1.52210536E-114, -1.30863065E-115, 2.59298855E-116, 6.07788997E-117, -9.25619074E-118, 8.34672868E-119, 1.70513549E-119, -5.91909049E-121, 3.19953920E-120, -3.70457274E-121, 1.24904433E-121, 8.45479982E-123, 7.91077884E-123, -1.79498494E-123, -6.83463448E-124, 1.35567792E-124, -5.54435799E-125},
        {-3.50843416E-008, -6.31707426E-010, -3.52212966E-012, -5.22811765E-014, -2.06383140E-015, 9.43474618E-018, 4.53896666E-019, -3.67283869E-021, -1.78534306E-022, -1.70825828E-025, 4.54136777E-027, 5.03553027E-028, 1.28430964E-030, 1.22676911E-031, 7.07279374E-033, -3.70928282E-035, 1.47841068E-036, 3.62034932E-039, 2.34940741E-041, -4.16309058E-042, 2.14073292E-044, 7.41858107E-046, -1.07435181E-047, 1.43419347E-050, -4.70079949E-052, -5.82090474E-053, -4.49317202E-055, 3.68365082E-057, -5.59791774E-058, 4.56433281E-061, -1.48011693E-061, -6.58108157E-064, -2.55332255E-065, -1.19054127E-066, 6.98222510E-069, 2.30582460E-070, 9.42799554E-073, 4.22086483E-074, 2.57679244E-076, 9.67564761E-078, 2.20328124E-079, -1.19617027E-080, -2.85903257E-083, -1.06668717E-083, -3.34540237E-085, -2.37020371E-087, 7.75501426E-090, 1.29145851E-090, 7.92136891E-092, 3.88058088E-094, 6.75033371E-097, 2.91175209E-097, -7.28448351E-099, -4.56654028E-100, 1.93148103E-101, -9.98456784E-103, -3.21673490E-107, -2.20747451E-106, 5.17730098E-108, 3.19438008E-109, 3.93097966E-110, -1.62718545E-111, -1.71719555E-112, -2.06402821E-114, -3.17067841E-116, 9.32674963E-117, -4.52760961E-118, -1.10448220E-118, -2.41953152E-119, 4.30613119E-120, 4.80630345E-121, 1.46854648E-121, 6.88272362E-122, -3.70656750E-124, -3.10077234E-123, -7.28470913E-124, -2.92574599E-124, -4.66356737E-125, -2.70001755E-125, 7.08360440E-126, -1.69751020E-126},
        {-4.23839215E-008, 9.37314545E-010, -3.57189165E-012, -1.03228258E-013, -3.43007687E-015, -3.63651982E-017, -4.52317143E-019, -1.19770756E-021, -4.29495744E-023, -1.77789438E-024, 2.32572414E-026, -7.08120362E-028, 8.45364032E-030, -2.12564173E-032, -9.02972901E-034, -2.19146514E-035, 3.62678963E-037, -4.82001771E-039, 1.21256638E-041, 6.73862577E-043, -5.13588940E-044, 1.50623125E-046, 8.49947579E-048, -2.39541058E-049, -4.23566837E-051, 1.51085007E-053, -3.44368860E-055, -2.64679839E-057, -1.08293668E-058, 6.11317153E-062, -7.72848263E-062, 1.01742600E-063, 7.24783786E-066, -2.79064637E-067, 4.83966360E-069, -3.13273905E-070, -9.14937902E-073, 2.87336299E-074, 6.54530593E-076, -5.80104531E-079, 5.52834809E-079, 1.57567472E-081, 8.60514470E-083, 8.39742807E-085, -1.01802449E-085, -5.53633908E-088, 2.20297910E-089, -6.11477425E-091, 6.13982264E-093, -3.43958307E-095, -1.71595358E-095, 1.06105035E-098, 7.81884492E-099, 1.46227381E-100, 1.53622884E-103, 7.48422263E-104, 2.47017261E-105, -9.84181551E-107, -1.35545692E-109, -4.14276074E-109, -7.24220733E-112, -3.77917760E-112, -6.77733548E-114, 6.23887265E-115, 1.22468756E-115, 4.17388259E-118, 7.87948307E-119, 3.61246159E-121, -5.65228613E-122, -4.82210534E-122, 1.99136859E-122, 1.10665590E-123, -2.59014194E-124, -5.79749678E-125, -1.51250084E-125, -4.89810892E-126, -5.23307982E-127, 9.26574278E-127, 3.12998731E-127, 4.64241760E-127, -6.43571516E-128},
        {-5.14705356E-009, -6.38213129E-010, -8.34201174E-013, -1.36467050E-013, 4.59291719E-016, -4.55067493E-017, 3.66085238E-019, -8.88586845E-021, 4.47657807E-023, 3.07144286E-025, 1.35713507E-026, -6.18596218E-028, 2.43880880E-031, -3.62545561E-032, -4.57865920E-033, 1.10410077E-035, -3.68916871E-037, -5.07911989E-039, -6.53988602E-041, -2.63365213E-042, -3.26696062E-044, 3.61892440E-046, 7.62432919E-048, 4.36395867E-050, -9.74852150E-052, 7.56626266E-054, -6.23242183E-055, -1.92746468E-057, -3.03252773E-060, -9.41106512E-061, 7.90833653E-062, 3.74715539E-064, -4.90164148E-066, 1.20488590E-067, 2.02905415E-069, -6.37737792E-071, -8.49353255E-073, 4.27710420E-074, -2.52419348E-076, -5.59956905E-078, -6.54062316E-080, 1.39897905E-081, 7.57792725E-083, 1.03393425E-084, 1.57280500E-086, -3.48608640E-088, 1.39067909E-089, 8.02980156E-092, 2.34447123E-093, -1.30970797E-095, -6.46752549E-096, 2.41093597E-098, -4.44223280E-100, -7.29344762E-101, 1.11657075E-102, -3.67869904E-104, -4.18678862E-106, -7.21392368E-108, -2.68752478E-109, 3.37563267E-110, 2.38554086E-112, -6.08047509E-113, 6.89292635E-115, 1.70585352E-117, 2.45570258E-117, 8.85085011E-119, -1.04519029E-120, -1.22556812E-121, -1.98447996E-122, 1.97483402E-123, 8.59881852E-124, 2.34790032E-124, -5.32065500E-125, -4.72957446E-126, 3.90398457E-127, -7.98140922E-127, 1.08097096E-127, -7.61008998E-128, -6.36120547E-129, -4.77330152E-129, 5.60357975E-130},
        {-2.83793304E-009, -3.14097467E-010, 8.65998902E-012, 9.12504502E-014, 1.24622697E-016, 7.04541429E-017, 3.93584857E-019, -2.33454155E-021, -3.56342291E-023, 2.27362343E-025, -8.93719173E-027, -1.12060874E-028, 5.59705020E-030, -1.35531782E-031, -2.03437156E-033, 1.71643690E-035, -2.80872284E-037, -3.10534214E-039, -2.91406875E-041, -9.18550959E-043, -1.05397385E-044, -4.16587847E-047, 5.95646699E-048, 1.01874774E-049, -2.11960767E-052, 1.88034091E-053, 4.16761864E-055, 6.67093360E-057, 3.17951178E-059, -1.69008038E-060, 3.59466114E-062, -4.23361545E-064, -3.54522856E-066, 8.09955381E-069, 8.80225573E-071, 3.45410982E-071, -4.72700572E-073, 2.61807221E-075, 1.07971944E-076, -1.22293454E-078, -7.99939136E-080, -7.78722572E-082, -4.54425374E-083, -1.11671250E-085, 4.02682205E-087, 6.99770664E-089, -5.47589568E-091, 1.69045277E-091, 3.53735407E-093, 2.02442160E-095, -1.17747490E-096, -1.43194259E-098, -4.31861850E-100, -1.24572207E-101, -1.36709867E-103, -3.31890449E-105, -1.47136656E-106, 5.10785460E-108, -1.17478578E-109, 5.98583452E-111, -6.21523165E-113, 1.49787277E-114, 8.33476694E-115, 9.82243707E-117, 5.13314217E-118, -4.34943098E-119, -5.35264852E-120, 3.51055689E-122, -1.26295181E-123, 4.03082716E-124, -6.63866071E-126, -9.59770032E-126, -1.94326036E-126, 1.72815129E-127, -3.28101576E-128, 5.73125708E-129, 2.02631919E-130, 9.13876567E-130, 3.12847917E-131, -1.20763391E-131, 5.85140792E-133},
        {-1.12773635E-009, -5.57674704E-010, -3.22416962E-013, 2.56449897E-014, -1.03216631E-015, 1.50248510E-017, -1.49916874E-019, -1.08570985E-020, 9.40548827E-023, 1.70622259E-024, 1.04468666E-026, 3.33550199E-028, -6.03274093E-030, 1.03167380E-031, 7.66619094E-034, 1.10808598E-035, 6.63136204E-038, -3.35211443E-039, -1.56424301E-041, -1.90734197E-043, 1.84679368E-044, -1.95104922E-046, -4.89203571E-048, 7.49468354E-050, 7.91608304E-052, 2.13940143E-053, 3.51437192E-055, 2.20786207E-057, -2.98405959E-059, -2.44223523E-061, 7.77639281E-063, -1.25614626E-064, -1.79339088E-066, -4.87824670E-068, 2.75954615E-070, -1.87753645E-071, -3.18757684E-073, -8.23318358E-075, 8.35363655E-077, -8.63090970E-079, -9.29346771E-081, -2.40037654E-083, -3.84492996E-084, 2.81251348E-085, -1.17169750E-087, -6.59891220E-089, -2.75501361E-090, -2.02382870E-092, 1.30357748E-093, 1.77319549E-096, 2.40328035E-097, 1.88249155E-098, 4.48526786E-100, 2.14379456E-102, 4.42343646E-104, 9.09569334E-107, -1.28161287E-106, 2.12983967E-108, -5.04388198E-110, 5.76549902E-112, 7.07861030E-113, 2.30620746E-114, 1.17517915E-115, 1.61899093E-117, 1.03829292E-119, -2.03999678E-120, 8.82530046E-122, -1.10596308E-122, 1.39178242E-124, 1.24030321E-125, 1.66543471E-126, 3.82504682E-127, 1.74612384E-128, 5.01829045E-129, -3.25091894E-130, -2.77948299E-130, -9.27653003E-131, 7.72713834E-132, -3.29335258E-132, 1.38349878E-132, 1.09033868E-133},
        {5.41092789E-009, 5.18083957E-011, -8.38102364E-012, -1.83863360E-013, 1.49956207E-015, -5.06929153E-018, 1.93597632E-019, 3.79442413E-022, -8.66320138E-023, 6.90717953E-025, -6.84224110E-027, 5.07066509E-028, 2.67914192E-030, 3.18808344E-032, -1.45877441E-034, 1.70776671E-035, 9.08408783E-038, -2.02072118E-039, -1.13220645E-041, 1.62047737E-042, 1.59522373E-044, -1.38980025E-046, -2.94004599E-048, -4.65433468E-050, 6.78868691E-052, -2.02219624E-054, -7.70470198E-056, -1.37133214E-057, 1.28668446E-059, 4.47603798E-061, 3.32032050E-063, 5.14750931E-065, -3.91010305E-066, 5.12285526E-068, -3.33905390E-070, -1.21245441E-071, 3.20899814E-073, -6.41595717E-075, -7.35563899E-077, -9.53958444E-079, 2.97544841E-081, 6.46576615E-082, -4.44179521E-084, 8.93179656E-086, 2.11893980E-087, -2.41161020E-089, -1.23872984E-090, -1.16931542E-092, -1.37763663E-094, -4.03888285E-097, -1.80723265E-097, 5.07722397E-099, -4.74707962E-101, -1.45356299E-102, -1.90179328E-104, 8.95520291E-107, 2.35050168E-107, 7.81930269E-109, 1.08478184E-110, 1.07914954E-111, -1.07580293E-113, 1.52453895E-116, 1.84641380E-116, 6.92150058E-119, -2.75585481E-119, 8.55220797E-121, 1.26750702E-122, 4.01158472E-123, -1.07500342E-124, -1.43416809E-126, 5.38301602E-128, -4.55320274E-129, -3.33087584E-129, 7.20978554E-131, 1.37385937E-130, -4.42743597E-132, 6.73407280E-133, 2.26597527E-133, 4.00431844E-136, -6.62549309E-134, 9.51132208E-135},
        {1.13653140E-008, 4.61891093E-010, -9.99081924E-012, 1.66180344E-013, -1.21318516E-015, 2.73253185E-017, 2.64539045E-019, 3.11250590E-021, -4.13166752E-023, -6.75253353E-025, 2.18001961E-027, -1.36880915E-028, 1.32608708E-030, 6.43050210E-033, 2.79251884E-034, 2.05446097E-037, -1.72615270E-037, 1.08615701E-039, -8.64506006E-042, 5.26465377E-043, 1.28790583E-044, -9.94065420E-047, -2.67731594E-048, -1.51084371E-050, 4.53274980E-052, -4.18946909E-054, -2.12657194E-055, -1.73078497E-057, -5.87954039E-060, -1.23606352E-061, -2.05311472E-063, -4.53625538E-065, 2.88640909E-067, 4.49891175E-068, -2.43297982E-070, -5.63685834E-072, 1.47866755E-073, -1.23461982E-075, -5.19614251E-077, -1.45168751E-079, -2.96787056E-081, -9.12986230E-083, -3.19510795E-084, -2.17723594E-086, 8.32945854E-088, 1.35972340E-089, -1.01634200E-091, 3.68417417E-093, -2.04288343E-095, 3.16706220E-096, -7.94565734E-099, 6.22187155E-100, -2.48324118E-101, -7.54107230E-103, 1.60490718E-104, 3.71900163E-106, -2.89013529E-108, 2.05667332E-109, -6.13491817E-111, -8.55691137E-113, -2.82765426E-115, 1.06200320E-115, -8.00332234E-118, 9.89465984E-119, -5.27824647E-120, 9.88324063E-122, -2.06996648E-122, 4.24177399E-125, -6.59077331E-126, 1.08069705E-126, -2.96721963E-129, 8.06649835E-130, -3.86344602E-132, -1.71731224E-132, 2.22331331E-133, -1.23701192E-133, 9.15798368E-134, -8.33043862E-135, -1.33288436E-135, 1.50749557E-135, 1.04533401E-136},
        {-2.03624641E-008, -8.07337213E-011, 4.10963588E-012, 7.11519384E-014, 1.48491904E-015, 3.71818892E-019, -1.76201573E-019, -6.05722468E-022, 7.06357562E-023, -1.40195826E-025, -2.25109122E-027, -1.32042679E-028, 8.61383631E-031, 2.60262959E-032, 7.66280307E-034, -6.73160756E-036, -4.81631455E-038, 4.20664255E-040, -3.79901768E-041, 7.97921325E-044, -9.34023953E-045, 5.32417425E-047, -7.82783662E-049, 1.06528842E-050, -1.61858957E-052, -2.13791144E-054, -6.51647380E-056, -6.04044506E-058, -1.33011779E-059, 8.81859629E-062, 1.09449966E-063, -4.60700289E-065, -2.83732213E-067, -2.30566634E-069, 1.69536681E-070, -8.42834577E-072, 3.96226316E-074, -1.11254527E-075, -6.88264931E-078, 4.77964694E-079, -4.11378381E-081, 1.48555786E-082, -6.84238095E-085, -3.29453933E-086, 2.26637086E-088, -6.05101912E-090, -1.72086947E-091, -3.93301016E-095, 3.55364386E-095, 2.36703370E-097, 5.53186480E-098, 2.87285392E-100, -1.21664892E-102, -3.22674309E-103, 5.08225031E-105, -9.93661463E-107, 2.56744666E-108, 7.71468020E-111, -1.71731767E-111, 8.43912118E-113, -3.26095002E-115, 1.20871002E-116, 4.58805055E-118, -2.86265620E-119, -6.60520683E-121, 1.14312385E-122, -8.67668998E-124, 1.68161935E-125, -2.87419332E-126, -1.19330820E-127, -2.15285073E-129, -3.16247273E-131, -5.94738394E-132, -4.86817846E-133, -6.00136592E-135, 5.63553463E-135, -4.85262301E-135, -4.02601858E-137, 1.14388661E-136, -2.73819012E-138, 1.43518620E-138},
        {-2.14828624E-008, -3.80306901E-010, -4.91877253E-012, 2.18922093E-014, 2.83171793E-016, -3.19431539E-017, 6.02076414E-019, 1.61036931E-021, 7.05218575E-023, -5.72535448E-025, 5.30227531E-027, 1.06256320E-028, -3.55028538E-030, 4.20108707E-032, 2.02984267E-034, -1.16031259E-036, -5.75723362E-038, 2.04195500E-041, -1.26213971E-041, -2.81050358E-043, -2.54468462E-045, -1.73375802E-047, -9.65774650E-050, -1.06118901E-050, 2.23291106E-053, -5.83502323E-055, 3.28651032E-056, 1.33402085E-058, -1.82708325E-060, 8.30761930E-062, -7.69476700E-064, 1.22780716E-065, 5.82552157E-067, -9.49780830E-069, 1.38365183E-070, -2.37771375E-072, -4.74243498E-074, 1.09530019E-076, -1.54156229E-078, 2.65614642E-079, -1.73783315E-082, 5.28253011E-083, -2.30861039E-085, -5.05517901E-087, 2.29663703E-088, 1.70441862E-090, -4.68144481E-092, -1.43200318E-094, -4.98193778E-095, 4.28004783E-097, 5.49174213E-099, 7.26695123E-101, -3.85160339E-102, -8.98569846E-105, 1.40316077E-105, 2.73894071E-107, -4.98387864E-109, -1.67065110E-110, 3.91598544E-113, -1.00836305E-113, -2.64318218E-115, -1.53531945E-116, -1.53651371E-118, 3.80856810E-120, -4.93152650E-122, -4.40115877E-123, 2.45897412E-124, -4.58780013E-126, 1.55050778E-128, -3.94857870E-129, -4.47822414E-130, -6.29114792E-131, 1.12791877E-133, -3.98220283E-134, -3.95520788E-135, 5.80539725E-136, -5.62179196E-137, -1.77429036E-137, 4.74085303E-138, 5.98485399E-140, 5.43750756E-140},
        {-2.65313434E-008, -1.83842798E-011, 5.31608678E-012, 7.25666600E-015, 7.61094573E-016, -9.02696440E-018, 3.49201764E-019, -1.77257140E-021, -4.69701031E-023, -9.27401806E-025, 4.16762892E-027, -8.02713992E-029, -1.70789771E-030, 4.19805985E-032, -1.78506644E-034, -7.65525230E-037, -1.11945991E-037, 1.49618517E-039, 1.90005311E-041, 3.88683930E-043, -1.92768371E-045, -3.72159840E-047, 1.25216909E-048, -4.32237515E-051, -1.42604641E-052, 1.26012137E-054, 5.50074973E-056, 3.44618317E-058, 4.06127770E-060, -1.08717273E-061, 1.22056586E-063, 4.72514957E-065, 1.85231462E-067, -7.29844695E-069, -2.64833250E-071, 9.86903580E-073, 1.73623427E-076, -2.45041162E-076, 4.14362018E-078, -4.00169725E-080, -9.72119990E-082, -3.48567602E-083, 1.27793134E-085, 1.60394758E-087, 1.09706477E-088, -5.57775486E-091, -8.96147944E-093, -8.26030810E-095, -1.75802047E-095, -1.14042337E-097, -1.87497398E-099, 1.46382584E-101, -2.27839185E-102, 3.50035342E-104, -1.29101215E-106, -5.06847408E-108, 7.13461911E-110, 1.08090400E-111, -1.10523226E-113, 7.84603713E-114, 6.98043489E-116, -2.90185618E-117, 9.96292906E-119, -4.96810687E-121, 1.27805803E-122, -8.34063592E-124, -1.73254729E-125, 5.56433828E-128, 2.46858386E-128, 4.32967858E-129, 7.20268136E-131, -3.21870815E-132, 2.71485987E-133, 8.64295709E-136, 4.49960079E-136, 4.55894610E-137, 2.87906227E-139, -3.08551769E-139, 1.91422243E-139, -1.42758727E-140, -3.51057030E-142},
        {1.93886744E-009, -5.96676181E-010, -2.77803899E-012, 1.62941126E-013, -1.25978068E-015, -3.05543181E-018, 2.29584709E-020, -2.24702181E-022, -9.20423855E-023, -3.43031766E-025, -2.00679260E-027, -9.81242945E-029, 1.29012072E-030, 1.53085189E-032, -1.48806584E-034, -5.68126272E-036, 3.96168467E-039, -7.55002245E-040, 8.06348762E-044, -7.88364420E-044, 3.34397910E-046, -2.86762446E-047, 4.82455189E-049, 9.08739529E-051, 1.46937444E-052, -8.48359108E-055, 1.39942481E-056, 2.61979738E-058, -1.55475610E-060, -7.74761750E-062, 8.67590851E-064, 2.38624304E-065, -1.33030042E-067, 9.04571384E-070, -5.66577002E-071, 2.01106420E-072, -3.39774713E-075, 5.57924279E-077, 2.71625560E-078, -4.47735129E-080, -7.77078180E-083, -1.54542859E-084, 3.06104527E-085, 3.93646084E-087, -1.41082243E-088, -2.39562830E-091, 4.50320015E-094, -2.70729131E-095, 7.64132083E-096, -1.21891262E-097, 1.92444386E-099, -2.00249658E-102, 2.23500441E-104, 1.36422854E-104, 7.16062342E-107, -1.01146174E-108, -3.88488875E-111, 8.74615376E-112, -2.03789627E-113, -8.80858949E-115, 1.34353256E-117, -2.51133883E-118, -1.79034520E-119, 8.12729473E-122, 5.57732489E-123, -7.79238894E-125, 1.56161560E-125, 3.67484979E-127, -5.93603603E-129, -3.69041668E-130, -5.65502392E-133, -4.47642028E-133, 3.29775033E-134, 1.75162171E-135, -2.55625236E-137, -5.41506257E-139, -1.20944743E-139, -2.16915502E-140, -2.98096959E-142, 2.08340095E-142, -3.00525105E-142},
        {-6.98771875E-010, -6.93462788E-010, 8.07941070E-012, -7.67348499E-015, -1.02572405E-015, 2.37349576E-018, -5.70774546E-020, -8.84982515E-022, -8.35073583E-023, 6.27146626E-025, 8.50250773E-027, -1.18546446E-028, 1.16482860E-030, 8.19464712E-033, -1.89303352E-034, -1.24095493E-036, 1.62956973E-038, -3.95923744E-040, -9.03076821E-042, -2.20545435E-044, 8.50482429E-046, 2.42445312E-047, 3.77609992E-050, 4.91267797E-051, -2.73003506E-053, -3.99199610E-056, -1.32016580E-056, -9.16705273E-059, -3.22062785E-060, 9.16555509E-063, -1.07579565E-064, -3.04841758E-066, 2.56463482E-067, 9.54227795E-070, 2.32183211E-071, 7.62658739E-073, 4.45498719E-077, -2.02785199E-076, 3.18700647E-078, 1.16977735E-081, -2.01793383E-082, 1.63846449E-083, 3.85695657E-087, 2.66932089E-087, -3.04575703E-089, -5.10248864E-091, -8.90842499E-093, -1.69855076E-095, -2.39578150E-097, -9.00092228E-098, 1.56683821E-100, 3.66552269E-102, 6.67363062E-103, 9.24242882E-105, 8.04716932E-107, -2.53244220E-108, -1.74401725E-110, -2.23012370E-112, -1.77685161E-113, 2.17784124E-115, -3.97136522E-117, -1.35887200E-118, 8.97367183E-120, 6.42230091E-122, 2.05447832E-123, 2.86866442E-125, -1.48982527E-126, 6.60348686E-129, -2.00846515E-129, 1.54499388E-131, 3.29648893E-132, -1.34266543E-133, 4.47339670E-135, -1.61414601E-136, 2.59886939E-138, 3.88646879E-140, -4.42614975E-141, 7.71547359E-143, -1.11948379E-142, 3.26727668E-143, 5.28471776E-144},
        {8.49697978E-009, 3.26002521E-010, -6.75606412E-012, -1.18155189E-014, 1.19902261E-015, 7.14711387E-018, 2.13996617E-019, 2.32111128E-021, 5.86087686E-023, 2.10676381E-025, 4.94505877E-027, -5.69695373E-029, -6.11441889E-031, 1.69310273E-032, -1.37876776E-034, 4.72938726E-038, -1.40593378E-038, 1.50730302E-040, 3.31544984E-042, 7.28328775E-045, 1.51682846E-045, -6.50631126E-048, -2.27843949E-049, -4.46156506E-052, -2.20253513E-053, -3.40712557E-055, -1.18006551E-056, -7.15874265E-059, -1.81446596E-060, -1.83555472E-062, -1.01414549E-064, 2.74109922E-066, 4.65448368E-068, -2.18670919E-070, 1.44624387E-071, 1.70390766E-073, 3.95683031E-076, 2.82188286E-077, 7.41950362E-079, -1.00323026E-081, -4.44791759E-083, 1.71335624E-084, -4.94972588E-086, 3.09647910E-088, 8.75700769E-090, -2.41366339E-091, 3.28845177E-093, -1.08114731E-095, -3.72472720E-097, 1.11050082E-098, -2.34380080E-100, -4.49281054E-102, 1.54409898E-103, -2.46118593E-105, -1.04349650E-107, -1.50942117E-108, 5.77363220E-111, 5.34548251E-113, -3.26917710E-114, 2.31449603E-116, -1.24191906E-117, -3.23622448E-119, -9.24851599E-121, -1.03333104E-121, -8.94388833E-125, -3.23902846E-125, 1.04058113E-128, -1.59330166E-128, 2.87633600E-130, 2.32823296E-131, 3.03016591E-133, -1.44636716E-134, -5.27430334E-136, -2.02691380E-137, -2.22922853E-138, 5.23691128E-140, 1.62326170E-141, 5.10445662E-143, 9.60334591E-144, 2.07940461E-145, -1.41962736E-145},
        {1.46549768E-008, 3.59518038E-010, -3.94558249E-013, -5.07918577E-014, 7.11415564E-016, 3.37641154E-018, 2.25099140E-019, 1.31824693E-021, 3.23670367E-023, -1.72972566E-026, 3.26338172E-027, -1.35785581E-029, 1.27818357E-030, -1.67510630E-033, 1.09283218E-034, 1.99725557E-036, 1.90549862E-038, 3.31239348E-040, -3.61117909E-042, 4.90414752E-044, 1.00429023E-045, 6.91765323E-048, -7.95151500E-050, -1.11004401E-051, 1.93175628E-054, 9.29970763E-055, 2.65588505E-057, -6.88869009E-059, 4.70789590E-061, -1.78846611E-062, -1.39794592E-064, -1.59866682E-066, -4.90053114E-068, 4.86008887E-071, 7.89520411E-072, -4.20376113E-074, -3.04946599E-076, 4.08465327E-077, 1.74442338E-079, 9.77878814E-081, -4.93098712E-083, -1.72840961E-084, -5.78711918E-086, -1.47020755E-088, -1.38004691E-090, -5.56597010E-092, 8.29607617E-094, 1.22853924E-095, 8.97855789E-097, 1.46687361E-098, -1.56956085E-100, -1.23707222E-102, -5.75686827E-104, -6.72267411E-106, 1.46436290E-107, -2.61431274E-109, 7.48314443E-112, -1.14293193E-112, -9.03093681E-115, -4.33032618E-117, -4.75387951E-118, -1.18559412E-119, 6.05889289E-121, -7.01472101E-123, -3.19993933E-125, 3.28353897E-126, -1.03554205E-128, -2.68736330E-129, -5.98267556E-131, -2.94340121E-132, 1.48183060E-134, 4.24858525E-135, 2.26990379E-137, 4.00627883E-138, -7.32572692E-140, 3.22114019E-140, -6.70233552E-142, -1.09323354E-144, -5.22814799E-145, -1.20226760E-146, 7.06141186E-147},
        {3.69056792E-008, 2.70103410E-010, 1.32551010E-012, -1.08956629E-013, 2.70859084E-016, 6.23201242E-018, -1.00817127E-019, -3.94502673E-022, 2.90440386E-023, 8.33567398E-026, -3.76935485E-027, 2.41697183E-029, 1.04322075E-030, 3.13714347E-037, 1.22910594E-034, -2.21053281E-036, -1.85113192E-038, -2.73172934E-040, -6.69986491E-043, -4.84050549E-044, -4.08678797E-046, 1.28265902E-047, -1.74366487E-049, 2.75699996E-051, 2.27522324E-053, 1.76593704E-055, 1.89186562E-057, -1.64342017E-059, 7.14187954E-061, 5.36626101E-063, 2.10605586E-064, -2.38394713E-066, -3.43032929E-068, 1.45730815E-070, -3.62296504E-072, -6.61529211E-074, 3.28679922E-076, -9.03377497E-079, -1.35790630E-079, -4.50828650E-082, 1.01930566E-083, 3.52032040E-085, -1.33814891E-086, -1.30263468E-089, 6.87304354E-091, -3.91851037E-092, -1.59645938E-094, -2.66788700E-095, 2.40073597E-097, 4.94576876E-099, 4.82554891E-101, 1.01660954E-102, 7.73021121E-107, 3.46256959E-107, -4.70746411E-108, -9.86653482E-110, -1.50146738E-111, -7.56123033E-113, -6.02348118E-115, -9.89108618E-117, -9.98541022E-119, 9.74666608E-121, -3.09013488E-122, -1.19558540E-123, 3.99929464E-125, -3.04138147E-126, -6.58817933E-128, 1.12502635E-129, 5.72246427E-132, 1.74639553E-132, -6.97957085E-135, -3.48639939E-136, -3.34688395E-137, -6.69185266E-139, 2.05332156E-140, 2.57710117E-141, -1.17946804E-144, -3.99463807E-144, -3.27604883E-146, 2.42306405E-147, 1.90600001E-148},
        {-4.11310088E-009, -3.17371106E-010, 7.84996488E-013, -1.15997539E-013, 8.31780950E-016, -1.12795560E-017, -2.54646536E-019, -1.77827439E-021, 1.80208535E-025, -5.27116913E-027, -1.71876963E-027, -2.68175478E-029, 6.13364990E-031, -8.58816763E-033, -3.24576772E-035, -9.33190836E-037, -4.39714286E-039, -4.82088253E-041, -5.41554856E-042, -6.24066745E-044, 1.53407273E-046, 2.46452213E-048, 1.20799042E-049, 3.37369946E-053, 9.01894416E-054, -1.15456245E-055, -4.59303676E-058, -3.72481325E-059, 9.92473228E-061, 8.15638167E-063, 6.90217714E-065, 9.59155250E-067, 2.34166710E-068, 8.33196456E-071, 1.16711212E-072, -6.38691723E-074, -1.23060424E-075, -7.75807832E-079, -1.17158669E-079, -6.67534420E-081, 2.92224658E-083, 6.55294590E-085, -7.14817752E-087, 1.11053698E-088, 1.45530901E-090, -1.29612876E-092, -1.30629231E-095, -2.95975479E-097, -3.28518930E-098, -5.28256857E-100, 2.85409520E-101, -1.24133973E-103, 2.46043715E-105, -1.81718908E-106, -5.51167414E-108, -1.71116046E-110, -5.38084187E-114, -1.24975351E-113, -2.95673785E-115, -3.92660440E-117, -3.92185336E-119, 1.06227768E-120, -4.61835004E-122, 1.67130133E-124, -2.71064447E-125, 4.02919342E-127, -6.95515635E-129, 2.04037130E-130, -3.36908486E-132, 2.01243543E-134, -1.83131690E-135, -5.07286835E-137, 3.21769195E-138, -2.08467664E-139, 8.45624278E-141, -9.55661165E-143, -1.46278832E-145, -4.79266819E-145, 6.31084749E-146, 1.40592624E-148, -4.78764990E-150},
        {2.51721791E-009, -1.95049777E-010, -5.12145405E-013, -5.07180083E-014, -6.11075677E-016, -1.49246859E-017, -2.11093529E-019, 2.17441228E-021, -2.04518427E-024, -1.02716170E-025, -3.91101883E-027, -9.64688241E-030, -5.65344246E-031, -9.41366450E-033, -9.07779266E-035, -1.48444052E-036, -3.41971383E-038, 1.44387525E-040, -4.14511705E-043, -5.74287179E-044, -2.72408518E-046, -1.75405131E-048, 9.54393031E-050, -9.53729911E-052, 1.13641591E-053, -2.44388486E-055, 1.48695655E-057, -2.06959606E-059, 6.77478544E-062, 1.81065943E-063, -4.53425796E-065, 1.07897305E-066, -3.36760291E-069, 2.32311990E-070, 2.12736247E-072, -4.79731179E-074, -3.12890363E-076, 1.05058432E-078, -8.51264895E-080, -6.02766955E-082, 3.14200432E-083, -1.73188414E-085, -3.75961476E-087, 6.36740039E-089, 6.51265162E-091, -3.56422133E-093, 1.22319587E-094, 4.94831417E-096, 1.75919494E-098, 6.73423044E-100, -9.40086741E-102, 3.82768631E-104, 1.50805148E-105, -5.67349081E-108, -1.67172304E-108, 1.90188021E-110, 1.40723166E-112, -2.60030174E-114, 1.58387359E-115, 3.95051672E-117, 3.78985005E-119, 3.21769729E-121, 5.38604008E-123, -2.78541183E-124, -6.80336056E-127, -5.67377709E-128, -8.83224143E-129, -3.69133967E-132, -1.83069899E-132, -4.87910630E-134, 1.81504278E-135, 2.64032064E-137, -1.43298796E-140, -3.06586999E-140, 4.21113556E-142, -1.38871619E-143, -5.09292928E-145, -6.72915617E-146, -2.23578275E-147, -2.91394419E-148, -1.96445244E-150},
        {-7.83022766E-009, 1.25392478E-010, -4.59885650E-012, 9.25348580E-014, -9.53404560E-016, 1.26382346E-018, 2.78850615E-020, -1.05416689E-022, -2.87336671E-023, 2.05467644E-026, 2.45533807E-027, -1.63844276E-029, -6.63265539E-031, -6.09433969E-034, 7.10015413E-035, -4.17554709E-037, -1.10257805E-039, -4.57062015E-041, 3.00864506E-043, 6.42872285E-045, 2.32900796E-046, -6.02925121E-048, 7.32156451E-050, -1.61589771E-052, 1.36561308E-053, -2.56918196E-056, 5.67092996E-058, 1.30826861E-059, -1.33802206E-061, -1.91833974E-063, -9.62395667E-065, 8.56366657E-067, -6.49497207E-070, -2.57022974E-071, -1.22535407E-072, 2.59564798E-075, 1.75979758E-076, 2.85153034E-078, 4.14004863E-080, -7.38901318E-083, 2.53752895E-084, -4.46689626E-086, 7.21220502E-088, 1.18559876E-089, -4.10332144E-091, -9.72949953E-094, 3.63988024E-095, -2.61094096E-096, 3.85455841E-098, 3.91475625E-100, -3.54284604E-103, 1.55349848E-104, -2.18452007E-105, -5.06125648E-108, 7.78926199E-110, 1.01061595E-110, 7.15772828E-113, -1.52180025E-114, -6.82179836E-117, 2.65553802E-118, -1.16462587E-119, 1.67502879E-121, -7.18881256E-123, -2.18860350E-124, 2.74679794E-126, -2.74595816E-128, 3.91626138E-130, 3.58988587E-131, -5.49798122E-134, 4.94945457E-136, -2.44532900E-136, -1.49970785E-137, -3.49185361E-139, 4.58008429E-141, 2.74479534E-142, 1.66537475E-143, -8.63529564E-146, 3.89448930E-147, -3.72147269E-148, 1.74121000E-150, -1.26046941E-150}};
        
        
}
